from spaceone.core.error import *


class ERROR_INVALID_AUTHENTICATION_TYPE(ERROR_INVALID_ARGUMENT):
    _message = "Invalid authentication type (LOCAL OR EXTERNAL). (authentication_type = {authentication_type})"



from spaceone.core.pygrpc import BaseAPI
from spaceone.api.identity.v2 import job_pb2, job_pb2_grpc
from spaceone.identity.service.job_service import JobService


class Job(BaseAPI, job_pb2_grpc.JobServicer):
    pb2 = job_pb2
    pb2_grpc = job_pb2_grpc

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)
        project_svc = JobService(metadata)
        project_svc.delete(params)
        return self.empty()

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)
        job_svc = JobService(metadata)
        response: dict = job_svc.get(params)
        return self.dict_to_message(response)

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)
        job_svc = JobService(metadata)
        response: dict = job_svc.list(params)
        return self.dict_to_message(response)

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)
        job_svc = JobService(metadata)
        response: dict = job_svc.stat(params)
        return self.dict_to_message(response)



from mongoengine import *
from spaceone.core.model.mongo_model import MongoModel


class Role(MongoModel):
    role_id = StringField(max_length=40, required=True, unique_with="domain_id")
    name = StringField(max_length=255, unique_with="domain_id")
    state = StringField(choices=("ENABLED", "DISABLED"), default="ENABLED")
    version = StringField(max_length=40, default=None, null=True)
    role_type = StringField(
        max_length=20,
        choices=(
            "DOMAIN_ADMIN",
            "WORKSPACE_OWNER",
            "WORKSPACE_MEMBER",
        ),
    )
    permissions = ListField(StringField(max_length=255), default=[])
    page_access = ListField(StringField(max_length=255), default=[])
    tags = DictField(default=None)
    is_managed = BooleanField(default=False)
    domain_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)
    updated_at = DateTimeField(auto_now=True)

    meta = {
        "updatable_fields": [
            "name",
            "state",
            "permissions",
            "page_access",
            "tags",
            "updated_at",
        ],
        "minimal_fields": ["role_id", "name", "role_type", "is_managed", "state"],
        "ordering": ["name"],
        "indexes": ["role_type", "domain_id"],
    }



from typing import Union, Literal, List
from pydantic import BaseModel, Field

__all__ = [
    "SchemaCreateRequest",
    "SchemaUpdateRequest",
    "SchemaDeleteRequest",
    "SchemaGetRequest",
    "SchemaSearchQueryRequest",
    "SchemaStatQueryRequest",
    "SchemaType"
]

SchemaType = Literal["SERVICE_ACCOUNT", "TRUSTED_ACCOUNT", "SECRET", "TRUSTING_SECRET"]


class SchemaCreateRequest(BaseModel):
    schema_id: str
    name: str
    schema_type: SchemaType
    data_schema: Union[dict, None] = Field(None, alias="schema")
    provider: str
    related_schemas: Union[List[str], None] = None
    options: Union[dict, None] = None
    tags: Union[dict, None] = None
    domain_id: str


class SchemaUpdateRequest(BaseModel):
    schema_id: str
    name: Union[str, None] = None
    data_schema: Union[dict, None] = Field(None, alias="schema")
    related_schemas: Union[List[str], None] = None
    options: Union[dict, None] = None
    tags: Union[dict, None] = None
    domain_id: str


class SchemaDeleteRequest(BaseModel):
    schema_id: str
    domain_id: str


class SchemaGetRequest(BaseModel):
    schema_id: str
    domain_id: str


class SchemaSearchQueryRequest(BaseModel):
    query: Union[dict, None] = None
    schema_id: Union[str, None] = None
    name: Union[str, None] = None
    schema_type: Union[SchemaType, None] = None
    provider: Union[str, None] = None
    related_schema_id: Union[str, None] = None
    is_managed: Union[bool, None] = None
    domain_id: str


class SchemaStatQueryRequest(BaseModel):
    query: dict = None
    domain_id: str



from datetime import datetime
from typing import Union, List, Literal
from pydantic import BaseModel

from spaceone.core import utils

from spaceone.identity.model.workspace_user.request import State, AuthType
from spaceone.identity.model.role_binding.response import RoleBindingResponse

__all__ = [
    "WorkspaceUserResponse",
    "WorkspaceUsersResponse",
    "UserSummaryResponse",
    "UsersSummaryResponse",
]

RoleType = Literal["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER", "USER"]


class WorkspaceUserResponse(BaseModel):
    user_id: Union[str, None] = None
    name: Union[str, None] = None
    state: Union[State, None] = None
    email: Union[str, None] = None
    auth_type: Union[AuthType, None] = None
    role_type: Union[RoleType, None] = None
    language: Union[str, None] = None
    timezone: Union[str, None] = None
    tags: Union[dict, None] = None
    role_binding_info: Union[RoleBindingResponse, None] = None
    domain_id: Union[str, None] = None
    created_at: Union[datetime, None] = None
    last_accessed_at: Union[datetime, None] = None

    def dict(self, *args, **kwargs):
        data = super().dict(*args, **kwargs)
        data["created_at"] = utils.datetime_to_iso8601(data["created_at"])
        data["last_accessed_at"] = utils.datetime_to_iso8601(data["last_accessed_at"])
        return data


class WorkspaceUsersResponse(BaseModel):
    results: List[WorkspaceUserResponse]
    total_count: int


class UserSummaryResponse(BaseModel):
    user_id: str
    name: str
    state: State


class UsersSummaryResponse(BaseModel):
    results: List[UserSummaryResponse]
    total_count: int



import logging
from typing import Union

from spaceone.core.service import *
from spaceone.core.service.utils import *

from spaceone.identity.model.role_binding.request import *
from spaceone.identity.model.role_binding.response import *
from spaceone.identity.manager.role_binding_manager import RoleBindingManager
from spaceone.identity.manager.role_manager import RoleManager
from spaceone.identity.manager.user_manager import UserManager
from spaceone.identity.manager.workspace_manager import WorkspaceManager
from spaceone.identity.error.error_role import *

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class RoleBindingService(BaseService):
    resource = "RoleBinding"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.role_binding_manager = RoleBindingManager()
        self.user_mgr = UserManager()

    @transaction(
        permission="identity:RoleBinding.write",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER"],
    )
    @convert_model
    def create(
            self, params: RoleBindingCreateRequest
    ) -> Union[RoleBindingResponse, dict]:
        """create role binding

         Args:
            params (RoleBindingCreateRequest): {
                'user_id': 'str',                   # required
                'role_id': 'str',                   # required
                'resource_group': 'str',            # required
                'workspace_id': 'str',              # injected from auth
                'domain_id': 'str'                  # injected from auth (required)
            }

        Returns:
            RoleBindingResponse:
        """

        rb_vo = self.create_role_binding(params.dict())
        return RoleBindingResponse(**rb_vo.to_dict())

    def create_role_binding(self, params: dict):
        user_id = params["user_id"]
        role_id = params["role_id"]
        resource_group = params["resource_group"]
        domain_id = params["domain_id"]
        workspace_id = params.get("workspace_id")

        # Check user
        user_vo = self.user_mgr.get_user(user_id, domain_id)

        # Check workspace
        if resource_group == "WORKSPACE":
            workspace_mgr = WorkspaceManager()
            workspace_mgr.get_workspace(workspace_id, domain_id)
        else:
            params["workspace_id"] = "*"
            workspace_id = "*"

        # Check role
        role_mgr = RoleManager()
        role_vo = role_mgr.get_role(role_id, domain_id)

        if resource_group == "DOMAIN":
            if role_vo.role_type != "DOMAIN_ADMIN":
                raise ERROR_NOT_ALLOWED_ROLE_TYPE(
                    request_role_id=role_vo.role_id,
                    request_role_type=role_vo.role_type,
                    supported_role_type="DOMAIN_ADMIN",
                )
            self.check_duplicate_domain_admin_role(
                domain_id, user_id, role_vo.role_type
            )
        else:
            if role_vo.role_type not in ["WORKSPACE_OWNER", "WORKSPACE_MEMBER"]:
                raise ERROR_NOT_ALLOWED_ROLE_TYPE(
                    request_role_id=role_vo.role_id,
                    request_role_type=role_vo.role_type,
                    supported_role_type=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
                )
            self.check_duplicate_workspace_role(domain_id, workspace_id, user_id)

        params["role_type"] = role_vo.role_type

        # Update user role type
        latest_role_type = self._get_latest_role_type(
            user_vo.role_type, role_vo.role_type
        )

        user_role_info = {"role_type": latest_role_type}
        if role_vo.role_type in ["DOMAIN_ADMIN"]:
            user_role_info.update({"role_id": role_vo.role_id})

        self.user_mgr.update_user_by_vo(user_role_info, user_vo)

        # Create role binding
        return self.role_binding_manager.create_role_binding(params)

    @transaction(
        permission="identity:RoleBinding.write",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER"],
    )
    @convert_model
    def update_role(
            self, params: RoleBindingUpdateRoleRequest
    ) -> Union[RoleBindingResponse, dict]:
        """update role of role binding

         Args:
            params (RoleBindingUpdateRoleRequest): {
                'role_binding_id': 'str',           # required
                'role_id': 'str',                   # required
                'workspace_id': 'str',              # injected from auth
                'domain_id': 'str',                 # injected from auth (required)
            }

        Returns:
            RoleBindingResponse:
        """

        request_user_id = self.transaction.get_meta("authorization.user_id")

        rb_vo = self.role_binding_manager.get_role_binding(
            params.role_binding_id, params.domain_id, params.workspace_id
        )

        self.check_self_update_and_delete(request_user_id, rb_vo.user_id)

        # Check role
        role_mgr = RoleManager()
        new_role_vo = role_mgr.get_role(params.role_id, params.domain_id)

        if rb_vo.role_type in ["WORKSPACE_OWNER", "WORKSPACE_MEMBER"]:
            if new_role_vo.role_type not in ["WORKSPACE_OWNER", "WORKSPACE_MEMBER"]:
                raise ERROR_NOT_ALLOWED_ROLE_TYPE(
                    request_role_id=new_role_vo.role_id,
                    request_role_type=new_role_vo.role_type,
                    supported_role_type=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
                )
            self.check_last_workspace_owner_role_binding(
                new_role_vo.role_type, rb_vo.workspace_id, rb_vo.domain_id
            )
        elif rb_vo.role_type == new_role_vo.role_type:
            self.check_last_domain_admin_role_binding(
                new_role_vo.role_type, rb_vo.domain_id
            )
        else:
            raise ERROR_NOT_ALLOWED_ROLE_TYPE(
                request_role_id=new_role_vo.role_id,
                request_role_type=new_role_vo.role_type,
                supported_role_type=[rb_vo.role_type],
            )

        user_vo = self.user_mgr.get_user(rb_vo.user_id, rb_vo.domain_id)

        latest_role_type = self._get_latest_role_type(
            user_vo.role_type, new_role_vo.role_type
        )

        user_role_info = {"role_type": latest_role_type}
        if latest_role_type and new_role_vo.role_type in ["DOMAIN_ADMIN"]:
            user_role_info.update({"role_id": new_role_vo.role_id})

        self.user_mgr.update_user_by_vo(user_role_info, user_vo)

        rb_vo = self.role_binding_manager.update_role_binding_by_vo(
            {"role_id": params.role_id, "role_type": new_role_vo.role_type}, rb_vo
        )

        return RoleBindingResponse(**rb_vo.to_dict())

    @transaction(
        permission="identity:RoleBinding.write",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER"],
    )
    @convert_model
    def delete(self, params: RoleBindingDeleteRequest) -> None:
        """delete role binding

         Args:
            params (RoleBindingDeleteRequest): {
                'role_binding_id': 'str',       # required
                'workspace_id': 'str',          # injected from auth
                'domain_id': 'str',             # injected from auth (required)
            }

        Returns:
            None
        """

        request_user_id = self.transaction.get_meta("authorization.user_id")

        rb_vo = self.role_binding_manager.get_role_binding(
            params.role_binding_id, params.domain_id, params.workspace_id
        )

        self.check_self_update_and_delete(request_user_id, rb_vo.user_id)

        if rb_vo.role_type == "DOMAIN_ADMIN":
            self.check_last_domain_admin_role_binding(None, rb_vo.domain_id)
        elif rb_vo.role_type == "WORKSPACE_OWNER":
            self.check_last_workspace_owner_role_binding(None, rb_vo.workspace_id, rb_vo.domain_id)

        # Update user role type
        remain_rb_vos = self.role_binding_manager.filter_role_bindings(
            domain_id=params.domain_id, user_id=rb_vo.user_id
        )

        latest_role_type = "USER"
        for remain_rb_vo in remain_rb_vos:
            if remain_rb_vo.role_binding_id == params.role_binding_id:
                continue

            latest_role_type = self._get_latest_role_type(
                latest_role_type, remain_rb_vo.role_type
            )

        user_role_info = {"role_type": latest_role_type}
        if latest_role_type == "USER":
            user_role_info.update({"role_id": None})

        user_vo = self.user_mgr.get_user(rb_vo.user_id, rb_vo.domain_id)

        self.user_mgr.update_user_by_vo(user_role_info, user_vo)
        self.role_binding_manager.delete_role_binding_by_vo(rb_vo)

    @transaction(
        permission="identity:RoleBinding.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @convert_model
    def get(self, params: RoleBindingGetRequest) -> Union[RoleBindingResponse, dict]:
        """get role binding

         Args:
            params (RoleBindingGetRequest): {
                'role_binding_id': 'str',       # required
                'workspace_id': 'list',         # injected from auth
                'domain_id': 'str',             # injected from auth (required)
            }

        Returns:
             RoleBindingResponse:
        """

        rb_vo = self.role_binding_manager.get_role_binding(
            params.role_binding_id, params.domain_id, params.workspace_id
        )

        return RoleBindingResponse(**rb_vo.to_dict())

    @transaction(
        permission="identity:RoleBinding.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @append_query_filter(
        [
            "role_binding_id",
            "user_id",
            "role_id",
            "workspace_id",
            "domain_id",
        ]
    )
    @append_keyword_filter(["role_binding_id", "user_id", "role_id"])
    @convert_model
    def list(
            self, params: RoleBindingSearchQueryRequest
    ) -> Union[RoleBindingsResponse, dict]:
        """list role bindings

        Args:
            params (RoleBindingSearchQueryRequest): {
                'query': 'dict (spaceone.api.core.v1.Query)',
                'role_binding_id': 'str',
                'role_type': 'str',
                'user_id': 'str',
                'role_id': 'str',
                'workspace_id': 'list',         # injected from auth
                'domain_id': 'str',             # injected from auth (required)
            }

        Returns:
            RoleBindingsResponse:
        """

        query = params.query or {}
        rb_vos, total_count = self.role_binding_manager.list_role_bindings(query)

        rbs_info = [rb_vo.to_dict() for rb_vo in rb_vos]
        return RoleBindingsResponse(results=rbs_info, total_count=total_count)

    @transaction(
        permission="identity:RoleBinding.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @append_query_filter(["workspace_id", "domain_id"])
    @append_keyword_filter(["role_binding_id", "user_id", "role_id"])
    @convert_model
    def stat(self, params: RoleBindingStatQueryRequest) -> dict:
        """stat role bindings

        Args:
            params (RoleBindingStatQueryRequest): {
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)', # required
                'workspace_id': 'list',     # injected from auth
                'domain_id': 'str',         # injected from auth (required)
            }

        Returns:
            dict: {
                'results': 'list',
                'total_count': 'int'
            }
        """

        query = params.query or {}
        return self.role_binding_manager.stat_role_bindings(query)

    def check_duplicate_domain_admin_role(
            self, domain_id: str, user_id: str, role_type: str
    ) -> None:
        rb_vos = self.role_binding_manager.filter_role_bindings(
            domain_id=domain_id,
            user_id=user_id,
            role_type=role_type,
        )

        if rb_vos.count() > 0:
            raise ERROR_DUPLICATED_ROLE_BINDING(role_type=role_type)

    def check_duplicate_workspace_role(
            self, domain_id: str, workspace_id: str, user_id: str
    ) -> None:
        rb_vos = self.role_binding_manager.filter_role_bindings(
            domain_id=domain_id, workspace_id=workspace_id, user_id=user_id
        )

        if rb_vos.count() >= 1:
            raise ERROR_DUPLICATED_WORKSPACE_ROLE_BINDING(
                allowed_role_type=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"]
            )

    def check_last_domain_admin_role_binding(self, new_role_type: Union[str, None], domain_id: str) -> None:

        user_ids = self._get_enabled_user_ids(domain_id)
        rb_vos = self.role_binding_manager.filter_role_bindings(
            domain_id=domain_id,
            role_type="DOMAIN_ADMIN",
            user_id=user_ids,
        )

        if rb_vos.count() == 1 and new_role_type != "DOMAIN_ADMIN":
            raise ERROR_LAST_DOMAIN_ADMIN_CANNOT_DELETE()

    def check_last_workspace_owner_role_binding(
            self, new_role_type: Union[str, None], workspace_id: str, domain_id: str
    ) -> None:

        user_ids = self._get_enabled_user_ids(domain_id)
        rb_vos = self.role_binding_manager.filter_role_bindings(
            domain_id=domain_id,
            workspace_id=workspace_id,
            user_id=user_ids,
            role_type="WORKSPACE_OWNER",
        )

        if rb_vos.count() == 1 and new_role_type != "WORKSPACE_OWNER":
            raise ERROR_LAST_WORKSPACE_OWNER_CANNOT_DELETE()

    def _get_enabled_user_ids(self, domain_id: str) -> list:
        user_vos = self.user_mgr.filter_users(
            domain_id=domain_id,
            state="ENABLED",
        )

        return [user_vo.user_id for user_vo in user_vos]

    @staticmethod
    def check_self_update_and_delete(requested_user_id: str, user_id: str) -> None:
        if user_id == requested_user_id:
            raise ERROR_NOT_ALLOWED_TO_UPDATE_OR_DELETE_ROLE_BY_SELF()

    @staticmethod
    def _get_latest_role_type(before: str, after: str) -> str:
        priority = {
            "DOMAIN_ADMIN": 1,
            "WORKSPACE_OWNER": 2,
            "WORKSPACE_MEMBER": 3,
            "USER": 4,
        }

        before_priority = priority.get(before, 4)
        after_priority = priority.get(after, 4)

        if before_priority < after_priority:
            return before
        else:
            if after in ["WORKSPACE_OWNER", "WORKSPACE_MEMBER"]:
                return "USER"

            return after



import logging

from spaceone.identity.error.error_authentication import *
from spaceone.identity.error.error_user import *
from spaceone.identity.error.error_mfa import *
from spaceone.identity.manager.external_auth_manager import ExternalAuthManager
from spaceone.identity.manager.domain_manager import DomainManager
from spaceone.identity.manager.user_manager import UserManager
from spaceone.identity.manager.mfa_manager.base import MFAManager
from spaceone.identity.manager.token_manager.base import TokenManager
from spaceone.identity.model.domain.database import Domain
from spaceone.identity.model.user.database import User

_LOGGER = logging.getLogger(__name__)


class MFATokenManager(TokenManager):
    domain: Domain = None
    auth_type = "MFA"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.domain_mgr = DomainManager()
        self.external_auth_mgr = ExternalAuthManager()
        self.user_mgr = UserManager()

    def authenticate(self, domain_id: str, **kwargs):
        credentials = kwargs.get("credentials", {})
        mfa_info = MFAManager.get_mfa_info(credentials)
        if mfa_info is None:
            raise ERROR_INVALID_CREDENTIALS()

        user_id = mfa_info.get("user_id")
        domain_id = mfa_info.get("domain_id")

        self.user: User = self.user_mgr.get_user(user_id, domain_id)
        self._check_user_state()

        if verify_code := kwargs.get("verify_code"):
            if MFAManager.check_mfa_verify_code(credentials, verify_code):
                self.is_authenticated = True
            else:
                raise ERROR_INVALID_CREDENTIALS()
        else:
            user_mfa = self.user.mfa.to_dict()
            mfa_email = user_mfa["options"].get("email")

            mfa_manager = MFAManager.get_manager_by_mfa_type(user_mfa.get("mfa_type"))
            mfa_manager.send_mfa_authentication_email(
                self.user.user_id, domain_id, mfa_email, self.user.language, credentials
            )
            raise ERROR_MFA_REQUIRED(user_id=mfa_email)

    def _check_user_state(self) -> None:
        if self.user.state not in ["ENABLED", "PENDING"]:
            raise ERROR_USER_STATE_DISABLED(user_id=self.user.user_id)
        if self.user.mfa is None or self.user.mfa.state != "ENABLED":
            raise ERROR_MFA_NOT_ENABLED(key="user_id", value=self.user.user_id)



import logging
from typing import Tuple, List
from jsonschema import validate, exceptions

from spaceone.core import cache
from spaceone.core.manager import BaseManager
from spaceone.identity.error.error_schema import (
    ERROR_SCHEMA_IS_NOT_DEFINED,
    ERROR_SCHEMA_ID_IS_NOT_DEFINED,
    ERROR_INVALID_PARAMETER,
)
from spaceone.identity.model.schema.database import Schema
from spaceone.identity.manager.managed_resource_manager import ManagedResourceManager

_LOGGER = logging.getLogger(__name__)


class SchemaManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.schema_model = Schema

    def create_schema(self, params: dict) -> Schema:
        def _rollback(vo: Schema):
            _LOGGER.info(f"[create_schema._rollback] Delete schema : {vo.schema}")
            vo.delete()

        schema_vo = self.schema_model.create(params)
        self.transaction.add_rollback(_rollback, schema_vo)

        return schema_vo

    def update_schema_by_vo(self, params: dict, schema_vo: Schema) -> Schema:
        def _rollback(old_data):
            _LOGGER.info(
                f'[update_schema._rollback] Revert Data : {old_data["schema"]}'
            )
            schema_vo.update(old_data)

        self.transaction.add_rollback(_rollback, schema_vo.to_dict())

        return schema_vo.update(params)

    @staticmethod
    def delete_schema_by_vo(schema_vo: Schema) -> None:
        schema_vo.delete()

    def get_schema(self, schema_id: str, domain_id: str) -> Schema:
        return self.schema_model.get(schema_id=schema_id, domain_id=domain_id)

    def filter_schemas(self, **conditions) -> List[Schema]:
        return self.schema_model.filter(**conditions)

    def list_schemas(self, query: dict, domain_id: str) -> Tuple[list, int]:
        self._create_managed_schema(domain_id)
        return self.schema_model.query(**query)

    def stat_schemas(self, query: dict) -> dict:
        return self.schema_model.stat(**query)

    @cache.cacheable(key="identity:managed-schema:{domain_id}:sync", expire=300)
    def _create_managed_schema(self, domain_id: str) -> bool:
        managed_resource_mgr = ManagedResourceManager()

        schema_vos = self.filter_schemas(domain_id=domain_id, is_managed=True)

        installed_schema_version_map = {}
        for schema_vo in schema_vos:
            installed_schema_version_map[schema_vo.schema_id] = schema_vo.version

        managed_schema_map = managed_resource_mgr.get_managed_schemas()

        for managed_schema, managed_schema_info in managed_schema_map.items():
            managed_schema_info["domain_id"] = domain_id
            managed_schema_info["is_managed"] = True

            if schema_version := installed_schema_version_map.get(managed_schema):
                if schema_version != managed_schema_info["version"]:
                    _LOGGER.debug(
                        f"[_create_managed_schema] update managed schema: {managed_schema}"
                    )
                    schema_vo = self.get_schema(managed_schema, domain_id)
                    self.update_schema_by_vo(managed_schema_info, schema_vo)
            else:
                _LOGGER.debug(
                    f"[_create_managed_schema] create new managed schema: {managed_schema}"
                )
                self.create_schema(managed_schema_info)

        return True

    def validate_data_by_schema(
            self, provider: str, domain_id: str, schema_type: str, data: dict
    ) -> None:
        schema_vos = self.filter_schemas(
            provider=provider, domain_id=domain_id, schema_type=schema_type
        )

        if len(schema_vos) > 0:
            try:
                validate(instance=data, schema=schema_vos[0].schema)
            except exceptions.ValidationError as e:
                raise ERROR_INVALID_PARAMETER(key="data", reason=e.message)
        else:
            raise ERROR_SCHEMA_IS_NOT_DEFINED(
                provider=provider, schema_type=schema_type
            )

    def validate_secret_data_by_schema_id(
            self, schema_id: str, domain_id: str, data: dict, schema_type: str
    ) -> None:
        schema_vos = self.filter_schemas(
            schema_id=schema_id,
            domain_id=domain_id,
            schema_type=schema_type,
        )

        if len(schema_vos) > 0:
            try:
                validate(instance=data, schema=schema_vos[0].schema)
            except exceptions.ValidationError as e:
                raise ERROR_INVALID_PARAMETER(key="secret_data", reason=e.message)
        else:
            raise ERROR_SCHEMA_ID_IS_NOT_DEFINED(
                schema_id=schema_id, schema_type=schema_type
            )



import logging
from datetime import datetime
from typing import Union

from spaceone.core.service import *
from spaceone.core.service.utils import *

from spaceone.identity.error.error_job import *
from spaceone.identity.manager.account_collector_plugin_manager import (
    AccountCollectorPluginManager,
)
from spaceone.identity.manager.job_manager import JobManager
from spaceone.identity.manager.project_manager import ProjectManager
from spaceone.identity.manager.project_group_manager import ProjectGroupManager
from spaceone.identity.manager.provider_manager import ProviderManager
from spaceone.identity.manager.schema_manager import SchemaManager
from spaceone.identity.manager.service_account_manager import ServiceAccountManager
from spaceone.identity.manager.secret_manager import SecretManager
from spaceone.identity.manager.trusted_account_manager import TrustedAccountManager
from spaceone.identity.manager.workspace_manager import WorkspaceManager
from spaceone.identity.model.project.database import Project
from spaceone.identity.model.project_group.database import ProjectGroup
from spaceone.identity.model.provider.database import Provider
from spaceone.identity.model.service_account.database import ServiceAccount
from spaceone.identity.model.trusted_account.database import TrustedAccount
from spaceone.identity.model.job.database import Job
from spaceone.identity.model.job.request import *
from spaceone.identity.model.job.response import *
from spaceone.identity.model.workspace.database import Workspace
from spaceone.identity.service.service_account_service import ServiceAccountService

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class JobService(BaseService):
    resource = "Job"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.job_mgr = JobManager()
        self.trusted_account_mgr = TrustedAccountManager()
        self.provider_mgr = ProviderManager()
        self.account_collector_plugin_mgr = AccountCollectorPluginManager()
        self.workspace_mgr = WorkspaceManager()
        self.service_account_mgr = ServiceAccountManager()
        self.project_mgr = ProjectManager()
        self.project_group_mgr = ProjectGroupManager()

    @transaction(exclude=["authentication", "authorization", "mutation"])
    def create_jobs_by_trusted_account(self, params):
        """Create jobs by trusted account
        Args:
            params (dict): {
                'params':
            }
        Returns:
            None:
        """

        current_hour = params.get("current_hour", datetime.utcnow().hour)

        # todo check provider sync condition
        for trusted_account_vo in self._get_all_schedule_enabled_trusted_accounts(
                current_hour
        ):
            try:
                self.created_service_account_job(trusted_account_vo, {})
            except Exception as e:
                _LOGGER.error(
                    f"[create_jobs_by_trusted_account] sync error: {e}", exc_info=True
                )

    @transaction(
        permission="identity:Job.write", role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER"]
    )
    @convert_model
    def delete(self, params: JobDeleteRequest) -> None:
        """Delete job
        Args:
            params (JobDeleteRequest): {
                'job_id': 'str',            # required
                'workspace_id': 'str',      # injected from auth (required)
                'domain_id': 'str'          # injected from auth (required)
            }
        Returns:
            None:
        """

        job_vo = self.job_mgr.get_job(
            params.domain_id,
            params.job_id,
            params.workspace_id,
        )

        self.job_mgr.delete_job_by_vo(job_vo)

    @transaction(
        permission="identity:Job.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER"],
    )
    @convert_model
    def get(self, params: JobGetRequest) -> JobResponse:
        """Get job
        Args:
            params (JobGetRequest): {
                'job_id': 'str',            # required
                'workspace_id': 'str',      # injected from auth (required)
                'domain_id': 'str'          # injected from auth (required)
            }
        Returns:
            JobInfo:
        """

        job_id = params.job_id
        workspace_id = params.workspace_id
        domain_id = params.domain_id

        job_vo = self.job_mgr.get_job(domain_id, job_id, workspace_id)

        return JobResponse(**job_vo.to_dict())

    @transaction(
        permission="identity:Job.read", role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER"]
    )
    @append_query_filter(
        [
            "job_id",
            "status",
            "trusted_account_id",
            "plugin_id",
            "workspace_id",
            "domain_id",
        ]
    )
    @append_keyword_filter(["job_id", "status"])
    @convert_model
    def list(self, params: JobSearchQueryRequest) -> Union[JobsResponse, dict]:
        """List jobs
        Args:
            params (JobSearchQueryRequest): {
                'query': 'dict',
                'job_id': 'str',
                'status': 'str',
                'trusted_account_id': 'str',
                'plugin_id': 'str',
                'workspace_id': 'str',
                'domain_id': 'str'          # injected from auth (required)
            }
        Returns:
            JobsResponse:
        """

        query = params.query or {}

        job_vos, total_count = self.job_mgr.list_jobs(query)
        jobs_info = [job_vo.to_dict() for job_vo in job_vos]

        return JobsResponse(results=jobs_info, total_count=total_count)

    @transaction(
        permission="identity:Job.read", role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER"]
    )
    @append_query_filter(["workspace_id", "domain_id"])
    @append_keyword_filter(["job_id"])
    @convert_model
    def stat(self, params: JobStatQueryRequest) -> dict:
        """Stat jobs
        Args:
            params (JobStatQueryRequest): {
                'domain_id': 'str' # injected from auth (required)
            }
        Returns:
            dict:
        """

        query = params.query or {}

        return self.job_mgr.stat_jobs(query)

    @transaction(exclude=["authentication", "authorization", "mutation"])
    def sync_service_accounts(self, params: dict) -> None:
        """Sync account data
        Args:
            params (dict): {
                    'job_id': 'str',
                    'trusted_account_id': 'str',
                    'trusted_secret_id': 'str',
                    'secret_data': 'dict',
                    'workspace_id': 'str',
                    'domain_id': 'str'
            }
        Returns:
            None:
        """

        trusted_secret_id = params["trusted_secret_id"]
        secret_data = params["secret_data"]
        trusted_account_id = params["trusted_account_id"]
        job_id = params["job_id"]
        workspace_id = params.get("workspace_id")
        domain_id = params["domain_id"]

        schema_mgr = SchemaManager()

        trusted_account_vo: TrustedAccount = (
            self.trusted_account_mgr.get_trusted_account(
                trusted_account_id, domain_id, workspace_id
            )
        )
        job_vo: Job = self.job_mgr.get_job(domain_id, job_id, workspace_id)
        schema_vo = schema_mgr.get_schema(trusted_account_vo.secret_schema_id, domain_id)
        provider_vo: Provider = self.provider_mgr.get_provider(trusted_account_vo.provider, domain_id)
        plugin_info = provider_vo.plugin_info.to_dict()

        provider = provider_vo.provider
        sync_options = trusted_account_vo.sync_options or {}
        related_schemas = schema_vo.related_schemas or []

        if self._is_job_failed(job_id, domain_id, job_vo.workspace_id):
            self.job_mgr.change_canceled_status(job_vo)
        else:
            self.job_mgr.change_in_progress_status(job_vo)
            synced_projects = []
            synced_service_accounts = []

            try:
                options = plugin_info.get("options", {})
                schema_id = plugin_info.get("schema_id")

                (
                    endpoint,
                    updated_version,
                ) = self.account_collector_plugin_mgr.get_account_collector_plugin_endpoint(
                    plugin_info, domain_id
                )

                self.account_collector_plugin_mgr.initialize(endpoint)
                start_dt = datetime.utcnow()

                is_canceled = False

                response = self.account_collector_plugin_mgr.sync(
                    endpoint, options, secret_data, domain_id, schema_id
                )

                for result in response.get("results", []):
                    location = self._get_location(result, trusted_account_vo.resource_group, sync_options)

                    if trusted_account_vo.resource_group == "DOMAIN":
                        if not sync_options.get("single_workspace_id"):
                            workspace_vo = self._create_workspace(domain_id, location.pop(0))
                        else:
                            workspace_vo = self.workspace_mgr.get_workspace(domain_id,
                                                                            sync_options.get("single_workspace_id"))
                        sync_workspace_id = workspace_vo.workspace_id

                    else:
                        sync_workspace_id = workspace_id

                    parent_group_id = None
                    for loc in location:
                        project_group_vo = self._create_project_group(domain_id, sync_workspace_id, loc,
                                                                      parent_group_id)
                        parent_group_id = project_group_vo.project_group_id

                    project_vo = self._create_project(domain_id, sync_workspace_id, result["name"], parent_group_id,
                                                      sync_options)
                    synced_projects.append(project_vo)
                    service_account_vo = self._create_service_account(result, project_vo, trusted_account_id,
                                                                      trusted_secret_id,
                                                                      provider, result["name"], related_schemas,
                                                                      sync_options)
                    synced_service_accounts.append(service_account_vo)

                if self._is_job_failed(job_id, domain_id, job_vo.workspace_id):
                    self.job_mgr.change_canceled_status(job_vo)
                    is_canceled = True

                if not is_canceled:
                    end_dt = datetime.utcnow()
                    _LOGGER.debug(
                        f"[sync_service_accounts] end job ({job_vo.job_id}): {end_dt}"
                    )
                    _LOGGER.debug(
                        f"[sync_service_accounts] total job time ({job_vo.job_id}): {end_dt - start_dt}"
                    )
                    self.job_mgr.change_success_status(job_vo)

            except Exception as e:
                # self._delete_synced_resources(synced_projects, synced_service_accounts)
                self.job_mgr.change_error_status(job_vo, e)
                _LOGGER.error(f"[sync_service_accounts] sync error: {e}", exc_info=True)

        self._close_job(
            job_id,
            domain_id,
            job_vo.workspace_id,
        )

    def created_service_account_job(
            self, trusted_account_vo: TrustedAccount, job_options: dict
    ) -> Union[Job, dict]:
        resource_group = trusted_account_vo.resource_group
        provider = trusted_account_vo.provider
        trusted_account_id = trusted_account_vo.trusted_account_id
        workspace_id = trusted_account_vo.workspace_id
        domain_id = trusted_account_vo.domain_id

        provider_vo = self.provider_mgr.get_provider(provider, domain_id)
        plugin_id = provider_vo.plugin_info["plugin_id"]
        ac_plugin_mgr = AccountCollectorPluginManager()
        endpoint = ac_plugin_mgr.get_account_collector_plugin_endpoint_by_vo(
            provider_vo
        )

        options = provider_vo.plugin_info.get("options")
        schema_id = trusted_account_vo.secret_schema_id

        ac_plugin_mgr.initialize(endpoint)

        try:
            trusted_secret_data = self._get_trusted_secret_data(
                trusted_account_vo.trusted_secret_id, domain_id
            )

            if trusted_secret_data:
                schema_mgr = SchemaManager()
                # Check secret_data by schema
                schema_mgr.validate_secret_data_by_schema_id(
                    schema_id, domain_id, trusted_secret_data, "TRUSTED_ACCOUNT"
                )
        except Exception as e:
            trusted_secret_data = {}
            _LOGGER.error(
                f"[created_trusted_account_job] get trusted secret error: {e}",
                exc_info=True,
            )

        # Add Job Options
        job_vo = self.job_mgr.create_job(
            resource_group,
            domain_id,
            workspace_id,
            trusted_account_id,
            plugin_id,
        )

        if self._check_duplicate_job(domain_id, trusted_account_id, job_vo):
            self.job_mgr.change_error_status(
                job_vo, ERROR_DUPLICATE_JOB(trusted_account_id=trusted_account_id)
            )
        else:
            self.job_mgr.push_job(
                {
                    "job_id": job_vo.job_id,
                    "trusted_account_id": trusted_account_id,
                    "trusted_secret_id": trusted_account_vo.trusted_secret_id,
                    "secret_data": trusted_secret_data,
                    "workspace_id": trusted_account_vo.workspace_id,
                    "domain_id": domain_id,
                }
            )
            try:
                pass
            except Exception as e:
                self.job_mgr.change_error_status(job_vo, e)

        return job_vo

    def _get_all_schedule_enabled_trusted_accounts(self, current_hour: int) -> list:
        query = {
            "filter": [
                {"k": "schedule.state", "v": "ENABLED", "o": "eq"},
                {"k": "schedule.hours", "v": [current_hour], "o": "in"},
            ]
        }
        (
            trusted_account_vos,
            total_count,
        ) = self.trusted_account_mgr.list_trusted_accounts(query)
        _LOGGER.debug(
            f"[_get_all_schedule_enabled_trusted_accounts] scheduled trusted accounts count (UTC {current_hour}: {total_count}"
        )
        return trusted_account_vos

    def _get_trusted_secret_data(self, trusted_secret_id: str, domain_id: str) -> dict:
        # todo: this method is internal method
        secret_mgr: SecretManager = self.locator.get_manager("SecretManager")
        if trusted_secret_id:
            secret_data = secret_mgr.get_trusted_secret_data(
                trusted_secret_id, domain_id
            )
        else:
            secret_data = {}

        return secret_data

    def _check_duplicate_job(
            self,
            domain_id: str,
            trusted_account_id: str,
            his_job_vo: Job,
    ) -> bool:
        query = {
            "filter": [
                {"k": "trusted_account_id", "v": trusted_account_id, "o": "eq"},
                {"k": "workspace_id", "v": his_job_vo.workspace_id, "o": "eq"},
                {"k": "domain_id", "v": domain_id, "o": "eq"},
                {"k": "status", "v": ["PENDING", "IN_PROGRESS"], "o": "in"},
                {"k": "job_id", "v": his_job_vo.job_id, "o": "not"},
            ]
        }

        job_vos, total_count = self.job_mgr.list_jobs(query)

        if total_count == 0:
            return True
        else:
            for job_vo in job_vos:
                self.job_mgr.make_canceled_by_vo(job_vo)
        return False

    def _is_job_failed(
            self, job_id: str, domain_id: str, workspace_id: str = None
    ) -> bool:
        job_vo: Job = self.job_mgr.get_job(domain_id, job_id, workspace_id)

        if job_vo.status in ["CANCELED", "FAILURE"]:
            return True
        else:
            return False

    def _close_job(
            self,
            job_id: str,
            domain_id: str,
            workspace_id: str = None,
    ):
        job_vo: Job = self.job_mgr.get_job(domain_id, job_id, workspace_id)
        if job_vo.status == "IN_PROGRESS":
            self.job_mgr.change_success_status(job_vo)
        elif job_vo.status == "FAILURE":
            self.job_mgr.update_job_by_vo({"finished_at": datetime.utcnow()}, job_vo)

    def _create_workspace(self, domain_id: str, name: str) -> Workspace:
        workspace_vos = self.workspace_mgr.filter_workspaces(domain_id=domain_id, name=name)
        if workspace_vos:
            workspace_vo = workspace_vos[0]
        else:
            workspace_vo = self.workspace_mgr.create_workspace(
                {
                    "domain_id": domain_id,
                    "name": name,
                }
            )
        return workspace_vo

    def _create_project_group(self, domain_id: str, workspace_id: str, name: str,
                              parent_group_id: str = None) -> ProjectGroup:
        query_filter = {
            "filter": [{"k": "domain_id", "v": domain_id, "o": "eq"},
                       {"k": "workspace_id", "v": workspace_id, "o": "eq"}, {"k": "name", "v": name, "o": "eq"}]
        }
        if parent_group_id:
            query_filter["filter"].append({"k": "parent_group_id", "v": parent_group_id, "o": "eq"})

        project_group_vos, _ = self.project_group_mgr.list_project_groups(query_filter)

        if project_group_vos:
            project_group_vo = project_group_vos[0]
        else:
            params = {
                "name": name,
                "domain_id": domain_id,
                "workspace_id": workspace_id,
            }
            if parent_group_id:
                params["parent_group_id"] = parent_group_id
            project_group_vo = self.project_group_mgr.create_project_group(params)

        return project_group_vo

    def _create_project(
            self,
            domain_id: str,
            workspace_id: str,
            name: str,
            parent_group_id: str = None,
            sync_options: dict = None,
            project_type: str = "PRIVATE",
    ) -> Project:
        params = {
            "name": name,
            "domain_id": domain_id,
            "workspace_id": workspace_id,
            "project_type": project_type,
        }

        if parent_group_id:
            params["parent_group_id"] = parent_group_id

        project_vos = self.project_mgr.filter_projects(**params)

        if project_vos:
            project_vo = project_vos[0]
            self.project_mgr.update_project_by_vo(params, project_vo)
        else:
            project_vo = self.project_mgr.create_project(params)
        return project_vo

    def _create_service_account(
            self,
            result: dict,
            project_vo: Project,
            trusted_account_id: str,
            trusted_secret_id: str,
            provider: str,
            name: str,
            related_schemas: list = None,
            sync_options: dict = None,
    ) -> ServiceAccount:
        secret_data = result.get("secret_data", {})
        data = result.get("data", {})
        secret_schema_id = result.get("secret_schema_id")
        tags = result.get("tags", {})

        if secret_data and secret_schema_id is None:
            raise ERROR_INVALID_PARAMETER(key="secret_schema_id")

        if secret_schema_id not in related_schemas:
            raise ERROR_INVALID_PARAMETER(key="secret_schema_id", reason=f"schema_id is not in {related_schemas}")

        service_account_vos = self.service_account_mgr.filter_service_accounts(
            domain_id=project_vo.domain_id, workspace_id=project_vo.workspace_id, project_id=project_vo.project_id,
            data=data
        )

        if service_account_vos:
            service_account_vo = service_account_vos[0]
            if service_account_vo.name != result["name"]:
                service_account_vo = self.service_account_mgr.update_service_account_by_vo(
                    {"name": name}, service_account_vo
                )
        else:
            params = {
                "name": name,
                "provider": provider,
                "data": data,
                "trusted_account_id": trusted_account_id,
                "project_id": project_vo.project_id,
                "workspace_id": project_vo.workspace_id,
                "domain_id": project_vo.domain_id,
                "tags": tags,
            }
            if secret_schema_id:
                params["schema_id"] = secret_schema_id

            service_account_vo = self.service_account_mgr.create_service_account(params)

        if secret_data:
            # Check secret_data by schema
            schema_mgr = SchemaManager()
            schema_mgr.validate_secret_data_by_schema_id(
                secret_schema_id,
                service_account_vo.domain_id,
                secret_data,
                "TRUSTING_SECRET",
            )

            secret_mgr = SecretManager()
            create_secret_params = {
                "name": f"{service_account_vo.service_account_id}-secret",
                "data": secret_data,
                "schema_id": secret_schema_id,
                "trusted_secret_id": trusted_secret_id,
                "resource_group": "PROJECT",
            }
            secret_info = secret_mgr.create_secret(create_secret_params)
            # Update secret_id in service_account_vo
            service_account_vo = self.service_account_mgr.update_service_account_by_vo(
                {"secret_id": secret_info["secret_id"]}, service_account_vo
            )
        return service_account_vo

    # todo: change
    def _delete_synced_resources(
            self, synced_projects: list, synced_service_accounts: list, synced_projects_groups: list = None
    ) -> None:
        for service_account_vo in synced_projects:
            self.service_account_mgr.delete_service_account_by_vo(service_account_vo)

        for project_vo in synced_service_accounts:
            self.project_mgr.delete_project_by_vo(project_vo)

    @staticmethod
    def _get_location(result: dict, resource_group: str, sync_options: dict) -> list:
        location = []
        if not sync_options.get("skip_project_group", False):
            location = result.get("location", [])
            if resource_group == "DOMAIN" and not location:
                raise ERROR_REQUIRED_PARAMETER(key="location", reason="location is required")

        return location



import logging
from typing import Tuple, Union

from mongoengine import QuerySet
from spaceone.core.manager import BaseManager

from spaceone.identity.model.app.database import App

_LOGGER = logging.getLogger(__name__)


class AppManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.app_model = App

    def create_app(self, params: dict) -> App:
        def _rollback(vo: App):
            _LOGGER.info(f"[create_app._rollback] Delete app: {vo.name} ({vo.app_id})")
            vo.delete()

        app_vo = self.app_model.create(params)
        self.transaction.add_rollback(_rollback, app_vo)

        return app_vo

    def update_app_by_vo(self, params: dict, app_vo: App) -> App:
        def _rollback(old_data):
            _LOGGER.info(
                f'[update_app._rollback] Revert Data: {old_data["name"]} ({old_data["app_id"]})'
            )
            app_vo.update(old_data)

        self.transaction.add_rollback(_rollback, app_vo.to_dict())

        return app_vo.update(params)

    def enable_app(self, app_vo: App) -> App:
        self.update_app_by_vo({"state": "ENABLED"}, app_vo)

        return app_vo

    def disable_app(self, app_vo: App) -> App:
        self.update_app_by_vo({"state": "DISABLED"}, app_vo)

        return app_vo

    @staticmethod
    def delete_app_by_vo(app_vo: App) -> None:
        app_vo.delete()

    def get_app(
        self,
        app_id: str,
        domain_id: str,
        workspace_id: Union[str, None] = None,
        service_account_id: Union[str, None] = None,
    ) -> App:
        conditions = {
            "app_id": app_id,
            "domain_id": domain_id,
        }

        if workspace_id:
            conditions["workspace_id"] = workspace_id

        if service_account_id:
            conditions["service_account_id"] = service_account_id

        return self.app_model.get(**conditions)

    def filter_apps(self, **conditions) -> QuerySet:
        return self.app_model.filter(**conditions)

    def list_apps(self, query: dict) -> Tuple[list, int]:
        return self.app_model.query(**query)

    def stat_apps(self, query: dict) -> dict:
        return self.app_model.stat(**query)



from datetime import datetime
from typing import Union, List
from pydantic import BaseModel
from spaceone.core import utils

from spaceone.identity.model.role.request import RoleType, State

__all__ = ["RoleResponse", "RolesResponse"]


class RoleResponse(BaseModel):
    role_id: Union[str, None] = None
    name: Union[str, None] = None
    state: Union[State, None] = None
    role_type: Union[RoleType, None] = None
    permissions: Union[List[str], None] = None
    page_access: Union[List[str], None] = None
    tags: Union[dict, None] = None
    is_managed: Union[bool, None] = None
    domain_id: Union[str, None] = None
    created_at: Union[datetime, None] = None
    updated_at: Union[datetime, None] = None

    def dict(self, *args, **kwargs):
        data = super().dict(*args, **kwargs)
        data["created_at"] = utils.datetime_to_iso8601(data["created_at"])
        data["updated_at"] = utils.datetime_to_iso8601(data["updated_at"])
        return data


class RolesResponse(BaseModel):
    results: List[RoleResponse]
    total_count: int



import logging
from spaceone.core.auth.jwt import JWTUtil
from spaceone.core import cache
from spaceone.core.manager import *
from spaceone.core import utils
from spaceone.identity.model.domain.database import Domain, DomainSecret
from spaceone.identity.manager.domain_manager import DomainManager

_LOGGER = logging.getLogger(__name__)


class DomainSecretManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.domain_secret_model = DomainSecret
        self.domain_mgr = DomainManager()

    def create_domain_secret(self, domain_vo: Domain) -> None:
        def _rollback(vo: DomainSecret):
            _LOGGER.info(
                f"[create_domain_secret._rollback] Delete domain secret: {vo.domain_id}"
            )
            vo.delete()

        # Generate Domain-secret
        secret = self._generate_domain_secret(domain_vo)

        # Generate Domain Key
        secret["domain_key"] = utils.random_string(16)

        domain_secret_vo: DomainSecret = self.domain_secret_model.create(secret)
        self.transaction.add_rollback(_rollback, domain_secret_vo)

    def delete_domain_secret(self, domain_id: str) -> None:
        domain_secret_vos = self.domain_secret_model.filter(domain_id=domain_id)
        domain_secret_vos.delete()
        cache.delete(f"identity:public-jwk:{domain_id}")
        cache.delete(f"identity:private-jwk:{domain_id}")
        cache.delete(f"identity:refresh-public-jwk:{domain_id}")
        cache.delete(f"identity:refresh-private-jwk:{domain_id}")

    @cache.cacheable(key="identity:public-jwk:{domain_id}", expire=600)
    def get_domain_public_key(self, domain_id: str) -> dict:
        domain_secret_vo: DomainSecret = self.domain_secret_model.get(
            domain_id=domain_id
        )
        return domain_secret_vo.pub_jwk

    @cache.cacheable(key="identity:private-jwk:{domain_id}", expire=600)
    def get_domain_private_key(self, domain_id: str) -> dict:
        domain_secret_vo: DomainSecret = self.domain_secret_model.get(
            domain_id=domain_id
        )
        return domain_secret_vo.prv_jwk

    @cache.cacheable(key="identity:refresh-public-jwk:{domain_id}", expire=600)
    def get_domain_refresh_public_key(self, domain_id: str) -> dict:
        domain_secret_vo: DomainSecret = self.domain_secret_model.get(
            domain_id=domain_id
        )
        return domain_secret_vo.refresh_pub_jwk

    @cache.cacheable(key="identity:refresh-private-jwk:{domain_id}", expire=600)
    def get_domain_refresh_private_key(self, domain_id: str) -> dict:
        domain_secret_vo: DomainSecret = self.domain_secret_model.get(
            domain_id=domain_id
        )
        return domain_secret_vo.refresh_prv_jwk

    @staticmethod
    def _generate_domain_secret(domain_vo: Domain) -> dict:
        private_jwk, public_jwk = JWTUtil.generate_jwk()
        refresh_private_jwk, refresh_public_jwk = JWTUtil.generate_jwk()
        data = {
            "pub_jwk": public_jwk,
            "prv_jwk": private_jwk,
            "refresh_pub_jwk": refresh_public_jwk,
            "refresh_prv_jwk": refresh_private_jwk,
            "domain_id": domain_vo.domain_id,
            "domain": domain_vo,
        }
        return data



from typing import Union, Literal
from pydantic import BaseModel

__all__ = [
    "JobDeleteRequest",
    "JobGetRequest",
    "JobSearchQueryRequest",
    "JobStatQueryRequest",
]


class JobDeleteRequest(BaseModel):
    job_id: str
    workspace_id: Union[str, None] = None
    domain_id: str


class JobGetRequest(BaseModel):
    job_id: str
    workspace_id: Union[str, None] = None
    domain_id: str


class JobSearchQueryRequest(BaseModel):
    query: Union[dict, None] = None
    job_id: Union[str, None] = None
    trusted_account_id: Union[str, None] = None
    plugin_id: Union[str, None] = None
    workspace_id: Union[str, None] = None
    domain_id: str


class JobStatQueryRequest(BaseModel):
    query: dict
    workspace_id: Union[str, None] = None
    domain_id: str



from mongoengine import *
from spaceone.core.model.mongo_model import MongoModel


class ServiceAccount(MongoModel):
    service_account_id = StringField(max_length=40, generate_id="sa", unique=True)
    name = StringField(max_length=255, unique_with=["domain_id", "workspace_id"])
    data = DictField(default=None)
    provider = StringField(max_length=40)
    options = DictField(default=None)
    tags = DictField(default=None)
    secret_schema_id = StringField(max_length=40)
    secret_id = StringField(max_length=40)
    app_id = StringField(max_length=40, null=True, default=None)
    trusted_account_id = StringField(max_length=40, null=True, default=None)
    project_id = StringField(max_length=40)
    workspace_id = StringField(max_length=40)
    domain_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)

    meta = {
        "updatable_fields": [
            "name",
            "data",
            "options",
            "tags",
            "secret_schema_id",
            "secret_id",
            "app_id",
            "trusted_account_id",
            "project_id",
        ],
        "minimal_fields": [
            "service_account_id",
            "name",
            "provider",
            "trusted_account_id",
            "project_id",
            "workspace_id",
        ],
        "change_query_keys": {"user_projects": "project_id"},
        "ordering": ["name"],
        "indexes": [
            "name",
            "provider",
            "secret_schema_id",
            "secret_id",
            "trusted_account_id",
            "project_id",
            "workspace_id",
            "domain_id",
        ],
    }



from mongoengine import *
from spaceone.core.model.mongo_model import MongoModel


class App(MongoModel):
    app_id = StringField(max_length=40, generate_id="app", unique=True)
    name = StringField(max_length=40, required=True)
    state = StringField(
        max_length=20, default="ENABLED", choices=("ENABLED", "DISABLED", "EXPIRED")
    )
    is_managed = BooleanField(default=False)
    tags = DictField(default=None)
    role_type = StringField(
        max_length=20,
        choices=(
            "DOMAIN_ADMIN",
            "WORKSPACE_OWNER",
        ),
    )
    client_id = StringField(max_length=40, default=None, null=True)
    role_id = StringField(max_length=40, required=True)
    service_account_id = StringField(max_length=40, default=None, null=True)
    resource_group = StringField(max_length=40, choices=("DOMAIN", "WORKSPACE"))
    workspace_id = StringField(max_length=40)
    domain_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)
    expired_at = DateTimeField(required=True)
    last_accessed_at = DateTimeField(default=None, null=True)

    meta = {
        "updatable_fields": [
            "name",
            "state",
            "client_id",
            "service_account_id",
            "tags",
            "expired_at",
            "last_accessed_at",
        ],
        "minimal_fields": [
            "app_id",
            "name",
            "state",
            "role_type",
            "role_id",
            "expired_at",
        ],
        "ordering": ["app_id"],
        "indexes": [
            "state",
            "role_type",
            "client_id",
            "role_id",
            "resource_group",
            "workspace_id",
            "domain_id",
        ],
    }



import logging
from typing import Tuple, List

from spaceone.core.manager import BaseManager
from spaceone.core.error import *

from spaceone.identity.service.user_service import UserService
from spaceone.identity.service.role_binding_service import RoleBindingService
from spaceone.identity.manager.role_binding_manager import RoleBindingManager
from spaceone.identity.manager.user_manager import UserManager
from spaceone.identity.model.user.database import User
from spaceone.identity.error.error_workspace_user import (
    ERROR_USER_NOT_EXIST_IN_WORKSPACE,
)

_LOGGER = logging.getLogger(__name__)


class WorkspaceUserManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.user_svc = UserService()
        self.rb_svc = RoleBindingService()
        self.rb_mgr = RoleBindingManager()
        self.user_mgr = UserManager()

    def create_workspace_user(self, params: dict) -> User:
        user_vo = self.user_svc.create_user(params)

        self.rb_svc.create_role_binding(
            {
                "resource_group": "WORKSPACE",
                "user_id": params["user_id"],
                "role_id": params["role_id"],
                "permission_group": "WORKSPACE",
                "workspace_id": params["workspace_id"],
                "domain_id": params["domain_id"],
            }
        )

        return user_vo

    def check_workspace_users(
            self, users: List[str], workspace_id: str, domain_id: str
    ) -> None:
        rb_vos = self.rb_mgr.filter_role_bindings(
            user_id=users,
            workspace_id=[workspace_id, "*"],
            role_type=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
            domain_id=domain_id,
        )

        existing_users = list(set([rb.user_id for rb in rb_vos]))
        not_existing_users = list(set(users) - set(existing_users))

        for user_id in not_existing_users:
            raise ERROR_USER_NOT_EXIST_IN_WORKSPACE(
                user_id=user_id, workspace_id=workspace_id
            )

    def get_workspace_user(
            self, user_id: str, workspace_id: str, domain_id: str
    ) -> dict:
        user_ids, user_rb_map = self._get_role_binding_map_in_workspace(
            workspace_id, domain_id
        )
        if user_id not in user_ids:
            raise ERROR_NOT_FOUND(key="user_id", value=user_id)

        user_vo = self.user_mgr.get_user(user_id, domain_id)
        user_info = user_vo.to_dict()
        user_info["role_binding_info"] = user_rb_map[user_id]

        return user_info

    def list_workspace_users(
            self, query: dict, domain_id: str, workspace_id: str, role_type: str = None
    ) -> Tuple[list, int]:
        user_ids, user_rb_map = self._get_role_binding_map_in_workspace(
            workspace_id, domain_id, role_type
        )
        query["filter"] = query.get("filter", [])
        query["filter"].append({"k": "user_id", "v": user_ids, "o": "in"})

        user_vos, total_count = self.user_mgr.list_users(query)

        users_info = []
        for user_vo in user_vos:
            user_info = user_vo.to_dict()

            if user_rb_map:
                user_info["role_binding_info"] = user_rb_map[user_vo.user_id]

            users_info.append(user_info)

        return users_info, total_count

    def stat_workspace_users(
            self, query: dict, domain_id: str, workspace_id: str
    ) -> dict:
        user_ids, user_rb_map = self._get_role_binding_map_in_workspace(
            workspace_id, domain_id
        )
        query["filter"] = query.get("filter", [])
        query["filter"].append({"k": "user_id", "v": user_ids, "o": "in"})

        return self.user_mgr.stat_users(query)

    def _get_role_binding_map_in_workspace(
            self, workspace_id: str, domain_id: str, role_type: str = None
    ) -> Tuple[list, dict]:
        user_rb_map = {}
        user_ids = []

        if role_type and role_type not in ["WORKSPACE_OWNER", "WORKSPACE_MEMBER"]:
            raise ERROR_INVALID_PARAMETER(message=f"role_type must be one of [WORKSPACE_OWNER, WORKSPACE_MEMBER]")

        if role_type is None:
            role_type = ["WORKSPACE_OWNER", "WORKSPACE_MEMBER"]

        rb_vos = self.rb_mgr.filter_role_bindings(
            domain_id=domain_id,
            workspace_id=[workspace_id, "*"],
            role_type=role_type,
        )

        for rb_vo in rb_vos:
            user_ids.append(rb_vo.user_id)
            user_rb_map[rb_vo.user_id] = rb_vo.to_dict()

        return user_ids, user_rb_map



from spaceone.core.pygrpc import BaseAPI
from google.protobuf.json_format import ParseDict
from spaceone.api.identity.v2 import trusted_account_pb2, trusted_account_pb2_grpc
from spaceone.api.identity.v2 import job_pb2
from spaceone.identity.service.trusted_account_service import TrustedAccountService


class TrustedAccount(BaseAPI, trusted_account_pb2_grpc.TrustedAccountServicer):
    pb2 = trusted_account_pb2
    pb2_grpc = trusted_account_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)
        trusted_account_svc = TrustedAccountService(metadata)
        response: dict = trusted_account_svc.create(params)
        return self.dict_to_message(response)

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)
        trusted_account_svc = TrustedAccountService(metadata)
        response: dict = trusted_account_svc.update(params)
        return self.dict_to_message(response)

    def update_secret_data(self, request, context):
        params, metadata = self.parse_request(request, context)
        trusted_account_svc = TrustedAccountService(metadata)
        response: dict = trusted_account_svc.update_secret_data(params)
        return self.dict_to_message(response)

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)
        trusted_account_svc = TrustedAccountService(metadata)
        trusted_account_svc.delete(params)
        return self.empty()

    def sync(self, request, context):
        params, metadata = self.parse_request(request, context)
        trusted_account_svc = TrustedAccountService(metadata)
        response: dict = trusted_account_svc.sync(params)
        return ParseDict(response, job_pb2.JobInfo())

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)
        trusted_account_svc = TrustedAccountService(metadata)
        response: dict = trusted_account_svc.get(params)
        return self.dict_to_message(response)

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)
        trusted_account_svc = TrustedAccountService(metadata)
        response: dict = trusted_account_svc.list(params)
        return self.dict_to_message(response)

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)
        trusted_account_svc = TrustedAccountService(metadata)
        response: dict = trusted_account_svc.stat(params)
        return self.dict_to_message(response)



from typing import Union, Literal, List
from pydantic import BaseModel

__all__ = [
    "UserSearchQueryRequest",
    "UserCreateRequest",
    "UserUpdateRequest",
    "UserVerifyEmailRequest",
    "UserStatQueryRequest",
    "UserSetRequiredActionsRequest",
    "UserDisableMFARequest",
    "UserDeleteRequest",
    "UserEnableRequest",
    "UserDisableRequest",
    "UserGetRequest",
    "AuthType",
    "State",
]

State = Literal["ENABLED", "DISABLED", "PENDING"]
AuthType = Literal["LOCAL", "EXTERNAL"]


class UserCreateRequest(BaseModel):
    user_id: str
    password: Union[str, None] = None
    name: Union[str, None] = ""
    email: Union[str, None] = ""
    auth_type: AuthType
    language: Union[str, None] = None
    timezone: Union[str, None] = None
    tags: Union[dict, None] = None
    reset_password: Union[bool, None] = False
    domain_id: str


class UserUpdateRequest(BaseModel):
    user_id: str
    password: Union[str, None] = None
    name: Union[str, None] = None
    email: Union[str, None] = None
    language: Union[str, None] = None
    timezone: Union[str, None] = None
    tags: Union[dict, None] = None
    reset_password: Union[bool, None] = None
    domain_id: str


class UserVerifyEmailRequest(BaseModel):
    user_id: str
    email: Union[str, None] = None
    domain_id: str


class UserDisableMFARequest(BaseModel):
    user_id: str
    domain_id: str


class UserSetRequiredActionsRequest(BaseModel):
    user_id: str
    required_actions: List[str]
    domain_id: str


class UserDeleteRequest(BaseModel):
    user_id: str
    domain_id: str


class UserEnableRequest(BaseModel):
    user_id: str
    domain_id: str


class UserDisableRequest(BaseModel):
    user_id: str
    domain_id: str


class UserGetRequest(BaseModel):
    user_id: str
    domain_id: str


class UserSearchQueryRequest(BaseModel):
    query: Union[dict, None] = None
    user_id: Union[str, None] = None
    name: Union[str, None] = None
    state: Union[State, None] = None
    email: Union[str, None] = None
    auth_type: Union[AuthType, None] = None
    # domain_id: str


class UserStatQueryRequest(BaseModel):
    query: dict
    domain_id: str



from mongoengine import *
from spaceone.core.model.mongo_model import MongoModel


class TrustedAccount(MongoModel):
    trusted_account_id = StringField(max_length=40, generate_id="ta", unique=True)
    name = StringField(max_length=255, unique_with=["domain_id"])
    data = DictField(default=None)
    provider = StringField(max_length=40)
    schedule = DictField(default=None, null=True)
    sync_options = DictField(default=None, null=True)
    tags = DictField(default=None)
    secret_schema_id = StringField(max_length=40)
    trusted_secret_id = StringField(max_length=40)
    resource_group = StringField(max_length=40, choices=("DOMAIN", "WORKSPACE"))
    workspace_id = StringField(max_length=40)
    domain_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)

    meta = {
        "updatable_fields": [
            "name",
            "data",
            "schedule",
            "sync_options",
            "tags",
            "secret_schema_id",
            "trusted_secret_id",
        ],
        "minimal_fields": [
            "trusted_account_id",
            "name",
            "provider",
            "sync_options",
            "secret_schema_id",
            "trusted_secret_id",
            "resource_group",
            "workspace_id",
        ],
        "ordering": ["name"],
        "indexes": [
            "provider",
            "secret_schema_id",
            "trusted_secret_id",
            "resource_group",
            "workspace_id",
            "domain_id",
        ],
    }



from google.protobuf.json_format import ParseDict
from spaceone.core.pygrpc import BaseAPI
from spaceone.api.identity.v2 import service_account_pb2, service_account_pb2_grpc, app_pb2
from spaceone.identity.service.service_account_service import (
    ServiceAccountService,
)


class ServiceAccount(BaseAPI, service_account_pb2_grpc.ServiceAccountServicer):
    pb2 = service_account_pb2
    pb2_grpc = service_account_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)
        service_account_svc = ServiceAccountService(metadata)
        response: dict = service_account_svc.create(params)
        return self.dict_to_message(response)

    def create_app(self, request, context):
        params, metadata = self.parse_request(request, context)
        service_account_svc = ServiceAccountService(metadata)
        response: dict = service_account_svc.create_app(params)
        return ParseDict(response, app_pb2.AppInfo())

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)
        service_account_svc = ServiceAccountService(metadata)
        response: dict = service_account_svc.update(params)
        return self.dict_to_message(response)

    def update_secret_data(self, request, context):
        params, metadata = self.parse_request(request, context)
        service_account_svc = ServiceAccountService(metadata)
        response: dict = service_account_svc.update_secret_data(params)
        return self.dict_to_message(response)

    def delete_secret_data(self, request, context):
        params, metadata = self.parse_request(request, context)
        service_account_svc = ServiceAccountService(metadata)
        response: dict = service_account_svc.delete_secret_data(params)
        return self.dict_to_message(response)

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)
        service_account_svc = ServiceAccountService(metadata)
        service_account_svc.delete(params)
        return self.empty()

    def enable_app(self, request, context):
        params, metadata = self.parse_request(request, context)
        service_account_svc = ServiceAccountService(metadata)
        response: dict = service_account_svc.enable_app(params)
        return ParseDict(response, app_pb2.AppInfo())

    def disable_app(self, request, context):
        params, metadata = self.parse_request(request, context)
        service_account_svc = ServiceAccountService(metadata)
        response: dict = service_account_svc.disable_app(params)
        return ParseDict(response, app_pb2.AppInfo())

    def regenerate_app(self, request, context):
        params, metadata = self.parse_request(request, context)
        service_account_svc = ServiceAccountService(metadata)
        response: dict = service_account_svc.regenerate_app(params)
        return ParseDict(response, app_pb2.AppInfo())

    def delete_app(self, request, context):
        params, metadata = self.parse_request(request, context)
        service_account_svc = ServiceAccountService(metadata)
        service_account_svc.delete_app(params)
        return self.empty()

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)
        service_account_svc = ServiceAccountService(metadata)
        response: dict = service_account_svc.get(params)
        return self.dict_to_message(response)

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)
        service_account_svc = ServiceAccountService(metadata)
        response: dict = service_account_svc.list(params)
        return self.dict_to_message(response)

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)
        service_account_svc = ServiceAccountService(metadata)
        response: dict = service_account_svc.stat(params)
        return self.dict_to_message(response)



from spaceone.identity.plugin.account_collector.lib.server import (
    AccountCollectorPluginServer,
)

app = AccountCollectorPluginServer()


@app.route("AccountCollector.init")
def account_collector_init(params: dict) -> dict:
    """init plugin by options

    Args:
        params (CollectorInitRequest): {
            'options': 'dict',    # Required
            'domain_id': 'str'
        }

    Returns:
        PluginResponse: {
            'metadata': 'dict'
        }
    """
    pass


@app.route("AccountCollector.sync")
def account_collector_sync(params: dict) -> dict:
    """AccountCollector sync

    Args:
        params (AccountCollectorInit): {
            'options': 'dict',          # Required
            'schema_id': 'str',
            'secret_data': 'dict',      # Required
            'domain_id': 'str'          # Required
        }

    Returns:
        AccountsResponse:
        {
            'results': [
                {
                    name: 'str',
                    data: 'dict',
                    secret_schema_id: 'str',
                    secret_data: 'dict',
                    tags: 'dict',
                    location: 'list'
                }
            ]
        }
    """
    pass



from mongoengine import *
from spaceone.core.model.mongo_model import MongoModel


class ProjectGroup(MongoModel):
    project_group_id = StringField(max_length=40, generate_id="pg", unique=True)
    name = StringField(max_length=40)
    tags = DictField(default=None)
    users = ListField(StringField(max_length=255), default=None)
    parent_group_id = StringField(max_length=40, null=True, default=None)
    workspace_id = StringField(max_length=40)
    domain_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)

    meta = {
        "updatable_fields": [
            "name",
            "tags",
            "users",
            "parent_group_id",
        ],
        "minimal_fields": [
            "project_group_id",
            "name",
            "parent_group_id",
            "workspace_id",
        ],
        "change_query_keys": {
            "user_id": "users",
        },
        "ordering": ["name"],
        "indexes": [
            "parent_group_id",
            "workspace_id",
            "domain_id",
        ],
    }



import logging

from spaceone.core.connector import BaseConnector
from spaceone.core.auth.jwt.jwt_util import JWTUtil

__all__ = ["AccountCollectorPluginConnector"]

_LOGGER = logging.getLogger(__name__)


class AccountCollectorPluginConnector(BaseConnector):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.client = None
        self.secret_data = None
        self.options = None
        self.schema = None
        token = self.transaction.get_meta("token")
        self.token_type = JWTUtil.get_value_from_token(token, "typ")

    def initialize(self, endpoint: str) -> None:
        static_endpoint = self.config.get("endpoint")

        if static_endpoint:
            endpoint = static_endpoint

        self.client = self.locator.get_connector(
            "SpaceConnector", endpoint=endpoint, token="NO_TOKEN"
        )

        self.secret_data = self.config.get("secret_data")
        self.options = self.config.get("options")
        self.schema = self.config.get("schema")

    def init(self, domain_id: str, options: dict):
        return self.client.dispatch(
            "AccountCollector.init", {"options": options, "domain_id": domain_id}
        )

    def sync(
            self,
            options: dict,
            secret_data: dict,
            schema: str,
            domain_id: str,
    ):
        params = {
            "options": self.options or options,
            "secret_data": self.secret_data or secret_data,
            "schema": self.schema or schema,
            "domain_id": domain_id,
        }
        return self.client.dispatch("AccountCollector.sync", params)



from spaceone.core.pygrpc.server import GRPCServer
from spaceone.identity.interface.grpc.system import System
from spaceone.identity.interface.grpc.domain import Domain
from spaceone.identity.interface.grpc.endpoint import Endpoint
from spaceone.identity.interface.grpc.external_auth import ExternalAuth
from spaceone.identity.interface.grpc.job import Job
from spaceone.identity.interface.grpc.project_group import ProjectGroup
from spaceone.identity.interface.grpc.project import Project
from spaceone.identity.interface.grpc.provider import Provider
from spaceone.identity.interface.grpc.role import Role
from spaceone.identity.interface.grpc.role_binding import RoleBinding
from spaceone.identity.interface.grpc.schema import Schema
from spaceone.identity.interface.grpc.service_account import ServiceAccount
from spaceone.identity.interface.grpc.app import App
from spaceone.identity.interface.grpc.token import Token
from spaceone.identity.interface.grpc.trusted_account import TrustedAccount
from spaceone.identity.interface.grpc.user_profile import UserProfile
from spaceone.identity.interface.grpc.user import User
from spaceone.identity.interface.grpc.user_group import UserGroup
from spaceone.identity.interface.grpc.workspace import Workspace
from spaceone.identity.interface.grpc.workspace_user import WorkspaceUser

_all_ = ["app"]

app = GRPCServer()
app.add_service(System)
app.add_service(Domain)
app.add_service(ExternalAuth)
app.add_service(Endpoint)
app.add_service(Workspace)
app.add_service(ProjectGroup)
app.add_service(Project)
app.add_service(Provider)
app.add_service(Schema)
app.add_service(TrustedAccount)
app.add_service(ServiceAccount)
app.add_service(Job)
app.add_service(Role)
app.add_service(RoleBinding)
app.add_service(UserProfile)
app.add_service(User)
app.add_service(WorkspaceUser)
app.add_service(UserGroup)
app.add_service(App)
app.add_service(Token)



import logging

from typing import Tuple, List
from spaceone.core import cache
from spaceone.core.auth.jwt import JWTAuthenticator, JWTUtil

from spaceone.core.service import *
from spaceone.core.service.utils import *

from spaceone.identity.error.error_authentication import *
from spaceone.identity.error.error_domain import ERROR_DOMAIN_STATE
from spaceone.identity.error.error_workspace import ERROR_WORKSPACE_STATE
from spaceone.identity.error.error_mfa import *
from spaceone.identity.manager.system_manager import SystemManager
from spaceone.identity.manager.domain_manager import DomainManager
from spaceone.identity.manager.domain_secret_manager import DomainSecretManager
from spaceone.identity.manager.mfa_manager.base import MFAManager
from spaceone.identity.manager.token_manager.base import TokenManager
from spaceone.identity.manager.user_manager import UserManager
from spaceone.identity.manager.app_manager import AppManager
from spaceone.identity.manager.role_binding_manager import RoleBindingManager
from spaceone.identity.manager.role_manager import RoleManager
from spaceone.identity.manager.project_manager import ProjectManager
from spaceone.identity.manager.project_group_manager import ProjectGroupManager
from spaceone.identity.manager.workspace_manager import WorkspaceManager
from spaceone.identity.model.token.request import *
from spaceone.identity.model.token.response import *
from spaceone.identity.model.user.database import User
from spaceone.identity.model.app.database import App

_LOGGER = logging.getLogger(__name__)


@event_handler
class TokenService(BaseService):
    resource = "Token"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.domain_mgr = DomainManager()
        self.domain_secret_mgr = DomainSecretManager()
        self.user_mgr = UserManager()
        self.app_mgr = AppManager()
        self.rb_mgr = RoleBindingManager()
        self.role_mgr = RoleManager()
        self.project_mgr = ProjectManager()
        self.project_group_mgr = ProjectGroupManager()
        self.workspace_mgr = WorkspaceManager()

    @transaction()
    @convert_model
    def issue(self, params: TokenIssueRequest) -> Union[TokenResponse, dict]:
        """Issue token
        Args:
            params (dict): {
                'credentials': 'dict',  # required
                'auth_type': 'str',     # required
                'timeout': 'int',
                'verify_code': 'str',
                'domain_id': 'str',     # required
            }
        Returns:
            TokenResponse:
        """

        domain_id = params.domain_id
        timeout = params.timeout
        verify_code = params.verify_code
        credentials = params.credentials

        private_jwk = self.domain_secret_mgr.get_domain_private_key(domain_id=domain_id)
        refresh_private_jwk = self.domain_secret_mgr.get_domain_refresh_private_key(
            domain_id=domain_id
        )

        # Check Domain state is ENABLED
        self._check_domain_state(domain_id)

        token_mgr = TokenManager.get_token_manager_by_auth_type(params.auth_type)
        token_mgr.authenticate(
            domain_id, verify_code=verify_code, credentials=credentials
        )

        user_vo = token_mgr.user
        user_mfa = user_vo.mfa.to_dict() if user_vo.mfa else {}
        permissions = self._get_permissions_from_required_actions(user_vo)

        if user_mfa.get("state", "DISABLED") == "ENABLED" and params.auth_type != "MFA":
            mfa_manager = MFAManager.get_manager_by_mfa_type(user_mfa.get("mfa_type"))
            mfa_email = user_mfa["options"].get("email")
            mfa_manager.send_mfa_authentication_email(
                user_vo.user_id, domain_id, mfa_email, user_vo.language, credentials
            )
            raise ERROR_MFA_REQUIRED(user_id=mfa_email)

        token_info = token_mgr.issue_token(
            private_jwk,
            refresh_private_jwk,
            domain_id,
            timeout=timeout,
            permissions=permissions,
        )

        return TokenResponse(**token_info)

    @transaction()
    @convert_model
    def grant(self, params: TokenGrantRequest) -> Union[GrantTokenResponse, dict]:
        """Grant token
        Args:
            params (dict): {
                'grant_type': 'str',    # required
                'token': 'str',         # required
                'scope': 'str',         # required
                'timeout': 'int',
                'workspace_id': 'str',
                'permissions': 'list',
            }
        Returns:
            GrantTokenResponse:
        """
        domain_id = self._extract_domain_id(params.token)
        timeout = params.timeout
        public_jwk = None  # todo: remove

        refresh_public_jwk = self.domain_secret_mgr.get_domain_refresh_public_key(
            domain_id=domain_id
        )

        # todo: remove
        if (
                domain_id == SystemManager.get_root_domain_id()
                and params.scope == "WORKSPACE"
        ):
            public_jwk = self.domain_secret_mgr.get_domain_public_key(domain_id)
            domain_id = params.domain_id

        if domain_id == SystemManager.get_root_domain_id() and params.scope != "SYSTEM":
            raise ERROR_PERMISSION_DENIED()
        elif params.scope == "WORKSPACE":
            if params.workspace_id is None:
                raise ERROR_REQUIRED_PARAMETER(key="workspace_id")

            self._check_workspace_state(params.workspace_id, domain_id)
            workspace_vo = self.workspace_mgr.get_workspace(
                params.workspace_id, domain_id
            )
            if workspace_vo.state != "ENABLED":
                raise ERROR_PERMISSION_DENIED()
        else:
            params.workspace_id = None

        # Check Domain state is ENABLED
        self._check_domain_state(domain_id)

        if public_jwk:
            # todo: remove
            decoded_token_info = self._verify_token(
                params.grant_type, params.token, public_jwk
            )
            role_id = "managed-workspace-owner"
            role_type = "WORKSPACE_OWNER"
            user_vo = None
        else:
            decoded_token_info = self._verify_token(
                params.grant_type, params.token, refresh_public_jwk
            )

            if decoded_token_info["owner_type"] != "USER":
                raise ERROR_PERMISSION_DENIED()

            user_vo = self.user_mgr.get_user(
                user_id=decoded_token_info["user_id"], domain_id=domain_id
            )
            role_type, role_id = self._get_user_role_info(
                user_vo, workspace_id=params.workspace_id
            )

        decoded_token_info["scope"] = params.scope
        decoded_token_info["workspace_id"] = params.workspace_id

        private_jwk = self.domain_secret_mgr.get_domain_private_key(domain_id=domain_id)
        refresh_private_jwk = self.domain_secret_mgr.get_domain_refresh_private_key(
            domain_id=domain_id
        )

        token_mgr = TokenManager.get_token_manager_by_auth_type("GRANT")
        app_id = None
        if public_jwk:
            # todo : remove
            token_mgr.is_authenticated = True
            token_mgr.owner_type = decoded_token_info.get("owner_type")
            app_id = decoded_token_info.get("app_id")
            token_mgr.role_type = role_type
        else:
            token_mgr.authenticate(
                domain_id,
                scope=params.scope,
                role_type=role_type,
                user_vo=user_vo,
            )

        if params.grant_type == "SYSTEM_TOKEN" and params.scope == "WORKSPACE":
            # todo : remove
            permissions = params.permissions
        elif role_id:
            permissions = self._get_role_permissions(role_id, domain_id)
        else:
            permissions = None

        if role_type == "WORKSPACE_MEMBER":
            user_projects = []
            project_groups = self.project_group_mgr.filter_project_groups(
                domain_id=domain_id,
                workspace_id=params.workspace_id,
                users=user_vo.user_id,
            )

            for project_group in project_groups:
                project_group_id = project_group.project_group_id
                user_projects.extend(self.project_group_mgr.get_projects_in_project_groups(domain_id, project_group_id))

            user_projects.extend(self._get_user_projects(
                user_vo.user_id, params.workspace_id, domain_id
            ))

            user_projects = list(set(user_projects))
        else:
            user_projects = None

        token_info = token_mgr.issue_token(
            private_jwk,
            refresh_private_jwk,
            domain_id,
            timeout=timeout,
            workspace_id=params.workspace_id,
            permissions=permissions,
            projects=user_projects,
            app_id=app_id,  # todo : remove
        )

        response = {
            "access_token": token_info["access_token"],
            "role_type": role_type,
            "role_id": role_id,
            "domain_id": domain_id,
            "workspace_id": params.workspace_id,
        }

        return GrantTokenResponse(**response)

    @cache.cacheable(
        key="identity:workspace-state:{domain_id}:{workspace_id}", expire=600
    )
    def _check_workspace_state(self, workspace_id: str, domain_id: str) -> None:
        workspace_vo = self.workspace_mgr.get_workspace(workspace_id, domain_id)

        if workspace_vo.state != "ENABLED":
            raise ERROR_WORKSPACE_STATE(workspace_id=workspace_id)

    @cache.cacheable(key="identity:domain-state:{domain_id}", expire=600)
    def _check_domain_state(self, domain_id: str) -> None:
        domain_vo = self.domain_mgr.get_domain(domain_id)

        if domain_vo.state != "ENABLED":
            raise ERROR_DOMAIN_STATE(domain_id=domain_vo.domain_id)

    @staticmethod
    def _get_permissions_from_required_actions(user_vo: User) -> Union[List[str], None]:
        if "UPDATE_PASSWORD" in user_vo.required_actions:
            return [
                "identity:UserProfile",
            ]

        return None

    @staticmethod
    def _extract_domain_id(token: str) -> str:
        try:
            decoded = JWTUtil.unverified_decode(token)
        except Exception as e:
            _LOGGER.error(f"[_extract_token] {e}")
            _LOGGER.error(token)
            raise ERROR_AUTHENTICATE_FAILURE(message="Cannot decode token.")

        domain_id = decoded.get("did")

        if domain_id is None:
            raise ERROR_AUTHENTICATE_FAILURE(message="Empty domain_id provided.")

        return domain_id

    @staticmethod
    def _verify_token(grant_type: str, token: str, public_jwk: dict) -> dict:
        try:
            decoded = JWTAuthenticator(public_jwk).validate(token)
        except Exception as e:
            _LOGGER.error(f"[_verify_refresh_token] {e}")
            raise ERROR_AUTHENTICATE_FAILURE(message="Token validation failed.")

        if decoded.get("typ") != grant_type:
            raise ERROR_INVALID_GRANT_TYPE(grant_type=grant_type)

        token_info = {
            "owner_type": decoded["own"],
            "token_id": decoded["jti"],
        }

        if token_info["owner_type"] == "USER":
            token_info["user_id"] = decoded["aud"]
        else:
            token_info["app_id"] = decoded["aud"]

        return token_info

    def _get_user_role_info(
            self, user_vo: User, workspace_id: str = None
    ) -> Tuple[str, Union[str, None]]:
        if user_vo.role_type == "DOMAIN_ADMIN":
            rb_vos = self.rb_mgr.filter_role_bindings(
                user_id=user_vo.user_id,
                domain_id=user_vo.domain_id,
                role_type=user_vo.role_type,
            )

            if rb_vos.count() > 0:
                return rb_vos[0].role_type, rb_vos[0].role_id

        else:
            rb_vos = self.rb_mgr.filter_role_bindings(
                user_id=user_vo.user_id,
                domain_id=user_vo.domain_id,
                role_type=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
                workspace_id=workspace_id,
            )

            if rb_vos.count() > 0:
                return rb_vos[0].role_type, rb_vos[0].role_id

        return "USER", None

    @staticmethod
    def _get_app_role_info(app_vo: App) -> Tuple[str, str]:
        return app_vo.role_type, app_vo.role_id

    @cache.cacheable(key="identity:role-permissions:{domain_id}:{role_id}", expire=600)
    def _get_role_permissions(self, role_id: str, domain_id: str) -> List[str]:
        role_vo = self.role_mgr.get_role(role_id=role_id, domain_id=domain_id)
        return role_vo.permissions

    def _get_user_projects(
            self, user_id: str, workspace_id: str, domain_id: str
    ) -> List[str]:
        user_projects = []

        public_project_vos = self.project_mgr.filter_projects(
            project_type="PUBLIC",
            domain_id=domain_id,
            workspace_id=workspace_id,
        )
        user_projects.extend(
            [project_vo.project_id for project_vo in public_project_vos]
        )

        user_project_vos = self.project_mgr.filter_projects(
            project_type="PRIVATE",
            domain_id=domain_id,
            users=user_id,
            workspace_id=workspace_id,
        )
        user_projects.extend([project_vo.project_id for project_vo in user_project_vos])

        return user_projects



import logging
from typing import Union

from spaceone.core.service import *
from spaceone.core.service.utils import *
from spaceone.core.auth.jwt import JWTAuthenticator

from spaceone.identity.manager.domain_manager import DomainManager
from spaceone.identity.manager.domain_secret_manager import DomainSecretManager
from spaceone.identity.manager.role_manager import RoleManager
from spaceone.identity.manager.role_binding_manager import RoleBindingManager
from spaceone.identity.manager.user_manager import UserManager
from spaceone.identity.manager.system_manager import SystemManager
from spaceone.identity.model.system.request import *
from spaceone.identity.model.system.response import *
from spaceone.identity.error.error_system import *
from spaceone.identity.error.error_domain import *

_LOGGER = logging.getLogger(__name__)


@event_handler
class SystemService(BaseService):
    resource = "System"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.domain_mgr = DomainManager()
        self.domain_secret_mgr = DomainSecretManager()
        self.user_mgr = UserManager()
        self.role_manager = RoleManager()

    @transaction()
    @convert_model
    def init(self, params: SystemInitRequest) -> Union[SystemResponse, dict]:
        """Init System
        Args:
            params (SystemInitRequest): {
                'admin': 'dict',    # required
                'force': 'bool'
            }
        Returns:
            SystemResponse:
        """

        system_mgr = SystemManager()

        root_domain_id = system_mgr.get_root_domain_id()
        root_domain_vos = self.domain_mgr.filter_domains(domain_id=root_domain_id)

        if root_domain_vos.count() == 0:
            _LOGGER.debug(f"[init] Create root domain: {root_domain_id}")
            root_domain_vo = self.domain_mgr.create_domain(
                {"domain_id": root_domain_id, "name": "root"}
            )

            self.domain_secret_mgr.delete_domain_secret(root_domain_vo.domain_id)
            self.domain_secret_mgr.create_domain_secret(root_domain_vo)

        else:
            if params.force is False:
                raise ERROR_SYSTEM_ALREADY_INITIALIZED()

            # Check System Token
            try:
                token = self.transaction.get_meta("token")
                root_pub_jwk = self.domain_secret_mgr.get_domain_public_key(
                    root_domain_id
                )
                JWTAuthenticator(root_pub_jwk).validate(token)
            except Exception as e:
                raise ERROR_UNKNOWN(message="Invalid System Token")

            root_domain_vo = root_domain_vos[0]

            _LOGGER.debug(
                f"[init] Reset root domain secret: {root_domain_vo.domain_id}"
            )
            self.domain_secret_mgr.delete_domain_secret(root_domain_vo.domain_id)
            self.domain_secret_mgr.create_domain_secret(root_domain_vo)

        if params.force:
            user_vos = self.user_mgr.filter_users(domain_id=root_domain_vo.domain_id)
            for user_vo in user_vos:
                _LOGGER.debug(
                    f"[init] Delete existing user in root domain: {user_vo.user_id}"
                )
                self.user_mgr.delete_user_by_vo(user_vo)

        # create admin user
        _LOGGER.debug(f"[init] Create admin user: {params.admin.user_id}")
        params_admin = params.admin.dict()
        params_admin["auth_type"] = "LOCAL"
        params_admin["domain_id"] = root_domain_vo.domain_id
        params_admin["role_type"] = "DOMAIN_ADMIN"
        params_admin["role_id"] = "managed-domain-admin"
        user_vo = self.user_mgr.create_user(params_admin, check_user_id=False)

        # create default role
        role_mgr = RoleManager()
        role_mgr.list_roles({}, root_domain_vo.domain_id)
        role_vos = self.role_manager.filter_roles(
            domain_id=root_domain_vo.domain_id, role_type="DOMAIN_ADMIN"
        )

        if len(role_vos) == 0:
            raise ERROR_DOMAIN_ADMIN_ROLE_IS_NOT_DEFINED()

        # create role binding
        _LOGGER.debug(
            f"[init] Create role binding: {user_vo.user_id} ({role_vos[0].role_id})"
        )
        role_binding_mgr = RoleBindingManager()
        params_rb = {
            "user_id": user_vo.user_id,
            "role_id": role_vos[0].role_id,
            "scope": "DOMAIN",
            "domain_id": user_vo.domain_id,
            "role_type": role_vos[0].role_type,
        }
        role_binding_mgr.create_role_binding(params_rb)

        system_token = system_mgr.create_system_token(
            root_domain_vo.domain_id, user_vo.user_id
        )

        response = {
            "domain_id": root_domain_vo.domain_id,
            "name": root_domain_vo.name,
            "system_token": system_token,
        }

        return SystemResponse(**response)



from datetime import datetime
from typing import Union, List
from pydantic import BaseModel
from spaceone.core import utils

from spaceone.identity.model.provider.request import UpgradeMode

__all__ = ["ProviderResponse", "ProvidersResponse"]


class Plugin(BaseModel):
    plugin_id: str = None
    version: Union[str, None] = None
    options: Union[dict, None] = None
    upgrade_mode: Union[UpgradeMode, None] = None


class ProviderResponse(BaseModel):
    provider: Union[str, None] = None
    name: Union[str, None] = None
    plugin_info: Union[Plugin, None] = None
    alias: Union[str, None] = None
    color: Union[str, None] = None
    icon: Union[str, None] = None
    order: Union[int, None] = None
    options: Union[dict, None] = None
    tags: Union[dict, None] = None
    is_managed: Union[bool, None] = None
    domain_id: Union[str, None] = None
    created_at: Union[datetime, None] = None
    updated_at: Union[datetime, None] = None

    def dict(self, *args, **kwargs):
        data = super().dict(*args, **kwargs)
        data["created_at"] = utils.datetime_to_iso8601(data["created_at"])
        data["updated_at"] = utils.datetime_to_iso8601(data["updated_at"])
        return data


class ProvidersResponse(BaseModel):
    results: List[ProviderResponse] = []
    total_count: int



import logging
from typing import Tuple
from mongoengine import QuerySet

from spaceone.core import cache
from spaceone.core.manager import BaseManager

from spaceone.identity.manager.role_binding_manager import RoleBindingManager
from spaceone.identity.model.workspace.database import Workspace

_LOGGER = logging.getLogger(__name__)


class WorkspaceManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.workspace_model = Workspace

    def create_workspace(self, params: dict) -> Workspace:
        def _rollback(vo: Workspace):
            _LOGGER.info(
                f"[create_workspace._rollback] Delete workspace : {vo.name} ({vo.workspace_id}) ({vo.domain_id})"
            )
            vo.delete()

        workspace_vo = self.workspace_model.create(params)
        self.transaction.add_rollback(_rollback, workspace_vo)

        return workspace_vo

    def update_workspace_by_vo(
            self, params: dict, workspace_vo: Workspace
    ) -> Workspace:
        def _rollback(old_data):
            _LOGGER.info(
                f'[update_workspace._rollback] Revert Data : {old_data["name"]} ({old_data["workspace_id"]})'
            )
            workspace_vo.update(old_data)

        self.transaction.add_rollback(_rollback, workspace_vo.to_dict())

        return workspace_vo.update(params)

    @staticmethod
    def delete_workspace_by_vo(workspace_vo: Workspace) -> None:
        rb_mgr = RoleBindingManager()
        rb_vos = rb_mgr.filter_role_bindings(
            workspace_id=workspace_vo.workspace_id, domain_id=workspace_vo.domain_id
        )

        if rb_vos.count() > 0:
            _LOGGER.debug(
                f"[delete_workspace_by_vo] Delete role bindings count with {workspace_vo.workspace_id} : {rb_vos.count()}")
            rb_vos.delete()

        workspace_vo.delete()

        cache.delete_pattern(
            f"identity:workspace-state:{workspace_vo.domain_id}:{workspace_vo.workspace_id}"
        )

    def enable_workspace(self, workspace_vo: Workspace) -> Workspace:
        self.update_workspace_by_vo({"state": "ENABLED"}, workspace_vo)
        cache.delete_pattern(
            f"identity:workspace-state:{workspace_vo.domain_id}:{workspace_vo.workspace_id}"
        )

        return workspace_vo

    def disable_workspace(self, workspace_vo: Workspace) -> Workspace:
        self.update_workspace_by_vo({"state": "DISABLED"}, workspace_vo)
        cache.delete_pattern(
            f"identity:workspace-state:{workspace_vo.domain_id}:{workspace_vo.workspace_id}"
        )

        return workspace_vo

    def get_workspace(self, workspace_id: str, domain_id: str) -> Workspace:
        return self.workspace_model.get(domain_id=domain_id, workspace_id=workspace_id)

    def filter_workspaces(self, **conditions) -> QuerySet:
        return self.workspace_model.filter(**conditions)

    def list_workspaces(self, query: dict) -> Tuple[QuerySet, int]:
        return self.workspace_model.query(**query)

    def stat_workspaces(self, query: dict) -> dict:
        return self.workspace_model.stat(**query)



import logging
import smtplib
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText

from spaceone.identity.error.error_smtp import *

from spaceone.core.connector import BaseConnector

__all__ = ["SMTPConnector"]

_LOGGER = logging.getLogger(__name__)


class SMTPConnector(BaseConnector):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.smtp = None
        host = self.config.get("host")
        port = self.config.get("port")
        user = self.config.get("user")
        password = self.config.get("password")
        self.from_email = self.config.get("from_email")
        self.set_smtp(host, port, user, password)

    def set_smtp(self, host, port, user, password):
        try:
            self.smtp = smtplib.SMTP(host, port)
            self.smtp.connect(host, port)
            self.smtp.ehlo()
            self.smtp.starttls()
            self.smtp.login(user, password)
        except Exception as e:
            _LOGGER.error(f"[set_smtp] set smtp failed : Please check smtp config {e}")
            raise ERROR_SMTP_CONNECTION_FAILED()

    def send_email(self, to_emails, subject, contents):
        multipart_msg = MIMEMultipart("alternative")

        multipart_msg["Subject"] = subject
        multipart_msg["From"] = self.from_email
        multipart_msg["To"] = to_emails

        multipart_msg.attach(MIMEText(contents, "html"))

        self.smtp.sendmail(
            self.from_email, to_emails.split(","), multipart_msg.as_string()
        )

    def quit_smtp(self):
        self.smtp.quit()



from typing import Union, Literal
from pydantic import BaseModel

__all__ = [
    "WorkspaceUserCreateRequest",
    "WorkspaceUserGetRequest",
    "WorkspaceUserFindRequest",
    "WorkspaceUserSearchQueryRequest",
    "WorkspaceUserStatQueryRequest",
    "AuthType",
    "State",
]

State = Literal["ENABLED", "DISABLED", "PENDING"]
AuthType = Literal["LOCAL", "EXTERNAL"]
PermissionGroup = Literal["DOMAIN", "WORKSPACE"]


class WorkspaceUserCreateRequest(BaseModel):
    user_id: str
    password: Union[str, None] = None
    name: Union[str, None] = ""
    email: Union[str, None] = ""
    auth_type: AuthType
    language: Union[str, None] = None
    timezone: Union[str, None] = None
    tags: Union[dict, None] = None
    reset_password: Union[bool, None] = False
    domain_id: str
    workspace_id: str
    role_id: str


class WorkspaceUserGetRequest(BaseModel):
    user_id: str
    domain_id: str
    workspace_id: str


class WorkspaceUserFindRequest(BaseModel):
    keyword: Union[str, None] = None
    state: Union[State, None] = None
    page: Union[dict, None] = None
    workspace_id: str
    domain_id: str


class WorkspaceUserSearchQueryRequest(BaseModel):
    query: Union[dict, None] = None
    user_id: Union[str, None] = None
    name: Union[str, None] = None
    state: Union[State, None] = None
    email: Union[str, None] = None
    auth_type: Union[AuthType, None] = None
    role_type: Union[str, None] = None
    domain_id: str
    workspace_id: str


class WorkspaceUserStatQueryRequest(BaseModel):
    query: dict
    domain_id: str
    workspace_id: str



from typing import Union, Literal
from pydantic import BaseModel

__all__ = [
    "UpgradeMode",
    "Plugin",
    "ProviderCreateRequest",
    "ProviderUpdateRequest",
    "ProviderUpdatePluginRequest",
    "ProviderDeleteRequest",
    "ProviderGetRequest",
    "ProviderSearchQueryRequest",
    "ProviderStatQueryRequest",
]

UpgradeMode = Literal["MANUAL", "AUTO"]


class Plugin(BaseModel):
    plugin_id: str = None
    version: Union[str, None] = None
    options: dict
    upgrade_mode: Union[UpgradeMode, None] = None


class ProviderCreateRequest(BaseModel):
    provider: str
    name: str
    alias: Union[str, None] = None
    plugin_info: Union[Plugin, None] = None
    color: Union[str, None] = None
    icon: Union[str, None] = None
    order: Union[int, None] = None
    options: Union[dict, None] = None
    tags: Union[dict, None] = None
    domain_id: str


class ProviderUpdateRequest(BaseModel):
    provider: str
    name: Union[str, None] = None
    alias: Union[str, None] = None
    color: Union[str, None] = None
    icon: Union[str, None] = None
    order: Union[int, None] = None
    options: Union[dict, None] = None
    tags: Union[dict, None] = None
    domain_id: str


class ProviderUpdatePluginRequest(BaseModel):
    provider: str
    version: Union[str, None] = None
    options: Union[dict, None] = None
    upgrade_mode: Union[str, None] = None
    domain_id: str


class ProviderDeleteRequest(BaseModel):
    provider: str
    domain_id: str


class ProviderGetRequest(BaseModel):
    provider: str
    domain_id: str


class ProviderSearchQueryRequest(BaseModel):
    query: Union[dict, None] = None
    provider: Union[str, None] = None
    name: Union[str, None] = None
    alias: Union[str, None] = None
    is_managed: Union[bool, None] = None
    domain_id: str


class ProviderStatQueryRequest(BaseModel):
    query: dict = None
    domain_id: str



import logging
from datetime import datetime
from typing import Tuple, Union

from mongoengine import QuerySet
from spaceone.core import queue, utils
from spaceone.core.error import *
from spaceone.core.manager import BaseManager

from spaceone.identity.model.job.database import Job

_LOGGER = logging.getLogger(__name__)


class JobManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.job_model = Job

    def create_job(
            self,
            resource_group: str,
            domain_id: str,
            workspace_id: str,
            trusted_account_id: str,
            plugin_id: str,
    ) -> Job:
        data = {
            "resource_group": resource_group,
            "plugin_id": plugin_id,
            "trusted_account_id": trusted_account_id,
            "domain_id": domain_id,
            "workspace_id": workspace_id,
        }
        job_vo = self.job_model.create(data)

        _LOGGER.debug(f"[create_job] create job: {job_vo.job_id}")
        return job_vo

    @staticmethod
    def delete_job_by_vo(job_vo: Job) -> None:
        _LOGGER.debug(f"[delete_job_by_vo] delete job: {job_vo.job_id}")
        job_vo.delete()

    def get_job(self, domain_id: str, job_id: str, workspace_id: str = None) -> Job:
        conditions = {"job_id": job_id, "domain_id": domain_id}

        if workspace_id:
            conditions["workspace_id"] = workspace_id

        return self.job_model.get(**conditions)

    def update_job_by_vo(self, params: dict, job_vo: Job) -> Job:
        def _rollback(old_data):
            _LOGGER.info(
                f'[update_project._rollback] Revert Data: {old_data["name"]} ({old_data["job_id"]})'
            )
            job_vo.update(old_data)

        self.transaction.add_rollback(_rollback, job_vo.to_dict())

        return job_vo.update(params)

    def filter_jobs(self, **conditions):
        return self.job_model.filter(**conditions)

    def list_jobs(self, query: dict) -> Tuple[QuerySet, int]:
        return self.job_model.query(**query)

    def stat_jobs(self, query: dict) -> dict:
        return self.job_model.stat(**query)

    def make_canceled_by_vo(self, job_vo: Job) -> None:
        _LOGGER.debug(f"[make_canceled_by_vo] cancel job: {job_vo.job_id}")
        self.update_job_by_vo(
            {"status": "CANCELED", "finished_at": datetime.utcnow()}, job_vo
        )

    def push_job(self, params: dict) -> None:
        token = self.transaction.meta.get("token")

        task = {
            "name": "sync_service_accounts",
            "version": "v1",
            "executionEngine": "BaseWorker",
            "stages": [
                {
                    "locator": "SERVICE",
                    "name": "JobService",
                    "metadata": {"token": token},
                    "method": "sync_service_accounts",
                    "params": {"params": params},
                }
            ],
        }
        _LOGGER.debug(f"[push_job] push job: {params}")

        queue.put("identity_q", utils.dump_json(task))

    @staticmethod
    def change_in_progress_status(job_vo: Job) -> None:
        _LOGGER.debug(f"[change_in_progress_status] start job: {job_vo.job_id}")
        return job_vo.update({"status": "IN_PROGRESS", "started_at": datetime.utcnow()})

    @staticmethod
    def change_success_status(job_vo: Job) -> None:
        _LOGGER.debug(f"[change_success_status] success job: {job_vo.job_id}")
        job_vo.update(
            {
                "status": "SUCCESS",
                "finished_at": datetime.utcnow(),
            }
        )

    @staticmethod
    def change_canceled_status(job_vo: Job) -> None:
        _LOGGER.debug(f"[change_cancel_status] job canceled: {job_vo.job_id}")
        job_vo.update({"status": "CANCELED", "finished_at": datetime.utcnow()})

    @staticmethod
    def change_error_status(
            job_vo: Job, error: Union[ERROR_BASE, Exception] = None
    ) -> None:
        if not isinstance(error, ERROR_BASE):
            error = ERROR_UNKNOWN(message=str(error))
        _LOGGER.debug(
            f"[change_error_status] job error ({job_vo.job_id}): {error.message}"
        )

        job_vo.update(
            {
                "status": "FAILURE",
                # "error": error.to_dict(),
                "finished_at": datetime.utcnow(),
            }
        )



from spaceone.core.pygrpc import BaseAPI
from spaceone.api.identity.v2 import role_binding_pb2, role_binding_pb2_grpc
from spaceone.identity.service.role_binding_service import RoleBindingService


class RoleBinding(BaseAPI, role_binding_pb2_grpc.RoleBindingServicer):
    pb2 = role_binding_pb2
    pb2_grpc = role_binding_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)
        role_binding_svc = RoleBindingService(metadata)
        response: dict = role_binding_svc.create(params)
        return self.dict_to_message(response)

    def update_role(self, request, context):
        params, metadata = self.parse_request(request, context)
        role_binding_svc = RoleBindingService(metadata)
        response: dict = role_binding_svc.update_role(params)
        return self.dict_to_message(response)

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)
        role_binding_svc = RoleBindingService(metadata)
        role_binding_svc.delete(params)
        return self.empty()

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)
        role_binding_svc = RoleBindingService(metadata)
        response: dict = role_binding_svc.get(params)
        return self.dict_to_message(response)

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)
        role_binding_svc = RoleBindingService(metadata)
        response: dict = role_binding_svc.list(params)
        return self.dict_to_message(response)

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)
        role_binding_svc = RoleBindingService(metadata)
        response: dict = role_binding_svc.stat(params)
        return self.dict_to_message(response)



from spaceone.core.error import *


class ERROR_APP_EXPIRED_LIMIT(ERROR_INVALID_ARGUMENT):
    _message = (
        "The maximum expiration limit for apps is 365 days. (expired_at = {expired_at})"
    )



from typing import Union, List, Literal
from pydantic import BaseModel

__all__ = [
    "RoleCreateRequest",
    "RoleUpdateRequest",
    "RoleEnableRequest",
    "RoleDisableRequest",
    "RoleDeleteRequest",
    "RoleGetRequest",
    "RoleSearchQueryRequest",
    "RoleStatQueryRequest",
    "RoleType",
    "State",
]

RoleType = Literal["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"]
State = Literal["ENABLED", "DISABLED"]


class RoleCreateRequest(BaseModel):
    name: str
    role_type: RoleType
    permissions: Union[List[str], None] = None
    page_access: Union[List[str], None] = None
    tags: Union[dict, None] = None
    domain_id: str


class RoleUpdateRequest(BaseModel):
    role_id: str
    name: Union[str, None] = None
    permissions: Union[List[str], None] = None
    page_access: Union[List[str], None] = None
    tags: Union[dict, None]
    domain_id: str


class RoleEnableRequest(BaseModel):
    role_id: str
    domain_id: str


class RoleDisableRequest(BaseModel):
    role_id: str
    domain_id: str


class RoleDeleteRequest(BaseModel):
    role_id: str
    domain_id: str


class RoleGetRequest(BaseModel):
    role_id: str
    domain_id: str


class RoleSearchQueryRequest(BaseModel):
    query: Union[dict, None] = None
    role_id: Union[str, None] = None
    name: Union[str, None] = None
    state: Union[State, None] = None
    role_type: Union[RoleType, None] = None
    domain_id: str


class RoleStatQueryRequest(BaseModel):
    query: dict
    domain_id: str



from spaceone.core.pygrpc.server import GRPCServer
from spaceone.core.plugin.server import PluginServer
from spaceone.identity.plugin.account_collector.interface.grpc import app
from spaceone.identity.plugin.account_collector.service.account_collector_service import (
    AccountCollectorService,
)

__all__ = ["AccountCollectorPluginServer"]


class AccountCollectorPluginServer(PluginServer):
    _grpc_app: GRPCServer = app
    _global_conf_path: str = (
        "spaceone.identity.plugin.account_collector.conf.global_conf:global_conf"
    )
    _plugin_methods = {
        "AccountCollector": {
            "service": AccountCollectorService,
            "methods": ["init", "sync"],
        }
    }



import logging
from typing import Tuple

from spaceone.core import config
from spaceone.core.manager import BaseManager

_LOGGER = logging.getLogger(__name__)


class EndpointManager(BaseManager):

    @staticmethod
    def list_endpoints(service: str = None) -> Tuple[list, int]:
        endpoints: list = config.get_global('ENDPOINTS', [])

        if service:
            endpoints: list = [endpoint for endpoint in endpoints if endpoint.get('service') == service]

        return endpoints, len(endpoints)



from mongoengine import *
from spaceone.core.model.mongo_model import MongoModel


class Schema(MongoModel):
    schema_id = StringField(max_length=40, unique_with='domain_id')
    name = StringField(max_length=40)
    version = StringField(max_length=40, default=None, null=True)
    schema_type = StringField(max_length=20, choices=('SERVICE_ACCOUNT', 'TRUSTED_ACCOUNT',
                                                      'SECRET', 'TRUSTING_SECRET'))
    schema = DictField(default=None)
    provider = StringField(max_length=40)
    related_schemas = ListField(StringField(max_length=40), default=None)
    options = DictField(default=None)
    tags = DictField(default=None)
    is_managed = BooleanField(default=False)
    domain_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)
    updated_at = DateTimeField(auto_now=True)

    meta = {
        'updatable_fields': [
            'name',
            'version',
            'schema',
            'related_schemas',
            'options',
            'tags',
            'updated_at'
        ],
        'minimal_fields': [
            'schema_id',
            'name',
            'schema_type',
            'provider',
            'is_managed',
            'version'
        ],
        'change_query_keys': {
            'related_schema_id': 'related_schemas',
        },
        'ordering': [
            'name'
        ],
        'indexes': [
            'schema_id',
            'schema_type',
            'provider',
            'related_schemas',
            'is_managed',
            'domain_id'
        ]
    }



from mongoengine import *
from spaceone.core.model.mongo_model import MongoModel


class ExternalAuth(MongoModel):
    domain_id = StringField(max_length=40, unique=True)
    state = StringField(max_length=20, default="ENABLED", choices=("ENABLED", "DISABLED"))
    plugin_info = DictField(default=None)
    updated_at = DateTimeField(auto_now=True)

    meta = {
        "updatable_fields": [
            "state",
            "plugin_info",
            "updated_at",
        ]
    }



import logging

from spaceone.core import config
from spaceone.core.manager import BaseManager
from spaceone.core.connector.space_connector import SpaceConnector

_LOGGER = logging.getLogger(__name__)

_AUTH_CONFIG_KEYS = ["settings"]


class ConfigManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.config_conn: SpaceConnector = self.locator.get_connector(
            "SpaceConnector", service="config"
        )

    def get_auth_config(self, domain_id: str) -> dict:
        system_token = config.get_global("TOKEN")
        params = {
            "query": {
                "filter": [
                    {"k": "name", "v": _AUTH_CONFIG_KEYS, "o": "in"},
                ]
            }
        }

        response = self.list_domain_configs(
            params, token=system_token, x_domain_id=domain_id
        )

        auth_config = {}
        for config_info in response.get("results", []):
            auth_config[config_info["name"]] = config_info["data"]

        return auth_config

    def list_domain_configs(
        self, params: dict, token: str = None, x_domain_id: str = None
    ) -> dict:
        return self.config_conn.dispatch(
            "DomainConfig.list",
            params,
            token=token,
            x_domain_id=x_domain_id,
        )
